@@ -1,24 +1,25 @@
 /***********************************************************************
   @C-file{
     author              = "Nelson H. F. Beebe",
-    version             = "2.11.4",
-    date                = "09 May 1998",
-    time                = "22:56:25 MDT",
+    version             = "3.03",
+    date                = "11 March 2018",
+    time                = "12:14:56 MDT",
     filename            = "bibclean.c",
-    address             = "Center for Scientific Computing
-			   University of Utah
-			   Department of Mathematics, 322 INSCC
+    address             = "University of Utah
+			   Department of Mathematics, 110 LCB
 			   155 S 1400 E RM 233
 			   Salt Lake City, UT 84112-0090
 			   USA",
     telephone           = "+1 801 581 5254",
-    FAX                 = "+1 801 585 1640, +1 801 581 4148",
+    FAX                 = "+1 801 581 4148",
     URL                 = "http://www.math.utah.edu/~beebe",
-    checksum            = "56587 2089 6885 52747",
+    checksum            = "56265 2241 7416 57389",
     email               = "beebe@math.utah.edu, beebe@acm.org,
-                           beebe@ieee.org (Internet)",
+			   beebe@computer.org (Internet)",
     codetable           = "ISO/ASCII",
     keywords            = "prettyprint, bibliography",
+    license             = "GNU General Public License, version 2 or
+                           later",
     supported           = "yes",
     docstring           = {Prettyprint one or more BibTeX files on stdin,
 			   or specified files, to stdout, and check
@@ -47,28 +48,42 @@
 
 			   Usage:
 
-			   bibclean [ -author ] [ -error-log filename ]
-				    [ -help ] [ '-?' ] [ -init-file filename ]
+			   bibclean [ -author ]
+				    [ -copyleft ]
+				    [ -copyright ]
+				    [ -error-log filename ]
+				    [ -help ]
+				    [ '-?' ]
+				    [ -init-file filename ]
+				    [ -ISBN-file filename ]
+				    [ -keyword-file filename ]
 				    [ -max-width # ]
 				    [ -[no-]align-equals ]
 				    [ -[no-]check-values ]
+				    [ -[no-]debug-match-failures]
 				    [ -[no-]delete-empty-values ]
 				    [ -[no-]file-position ]
 				    [ -[no-]fix-font-changes ]
-				    [ -[no-]fix-initials ] [ -[no-]fix-names ]
+				    [ -[no-]fix-initials ]
+				    [ -[no-]fix-math ]
+				    [ -[no-]fix-names ]
 				    [ -[no-]German-style ]
 				    [ -[no-]keep-linebreaks]
 				    [ -[no-]keep-parbreaks]
 				    [ -[no-]keep-preamble-spaces]
 				    [ -[no-]keep-spaces]
 				    [ -[no-]keep-string-spaces]
-				    [ -[no-]parbreaks ] [ -[no-]prettyprint ]
+				    [ -[no-]parbreaks ]
+				    [ -[no-]prettyprint ]
+				    [ -[no-]print-ISBN-table ]
+				    [ -[no-]print-keyword-table ]
 				    [ -[no-]print-patterns ]
 				    [ -[no-]read-init-files ]
 				    [ -[no-]remove-OPT-prefixes ]
 				    [ -[no-]scribe ]
 				    [ -[no-]trace-file-opening ]
-				    [ -[no-]warnings ] [ -version ]
+				    [ -version ]
+				    [ -[no-]warnings ]
 				    <infile or  bibfile1 bibfile2 bibfile3 ...
 				    >outfile
 
@@ -217,62 +232,24 @@ function declaration.
 ***********************************************************************/
 
 #include <config.h>
-#include <stdio.h>
 #include "xtypes.h"
+#include "xstdbool.h"
 #include "xstdlib.h"
 #include "xstring.h"
 #include "xctype.h"
 #include "xlimits.h"
+#include "xstat.h"
 #include "xunistd.h"
 
-RCSID("$Id: bibclean.c,v 1.12 1999/05/20 13:32:49 beebe Exp beebe $")
+RCSID("$Id: bibclean.c,v 1.21 2014/04/03 18:00:15 beebe Exp beebe $")
 
 #include "ch.h"
 #include "delete.h"
+#include "isbn.h"
 #include "keybrd.h"
 #include "yesorno.h"
 #include "match.h"			/* must come AFTER yesorno.h */
-
-typedef struct s_option_function_entry
-{
-    const char *name;		/* option name */
-    size_t min_match;		/* minimum length string match */
-    void (*function)(VOID);	/* function to call when option matched */
-} OPTION_FUNCTION_ENTRY;
-
-typedef struct s_name_pair
-{
-    const char *old_name;
-    const char *new_name;
-} NAME_PAIR;
-
-typedef struct s_position
-{
-    const char *filename;
-    long byte_position;
-    long last_column_position;
-    long column_position;
-    long line_number;
-} POSITION;
-
-typedef struct s_io_pair
-{
-    POSITION input;
-    POSITION output;
-} IO_PAIR;
-
-typedef struct s_pattern_table
-{
-    MATCH_PATTERN *patterns;
-    int current_size;
-    int maximum_size;
-} PATTERN_TABLE;
-
-typedef struct s_pattern_names
-{
-    const char *name;
-    PATTERN_TABLE *table;
-} PATTERN_NAMES;
+#include "typedefs.h"			/* must come AFTER match.h */
 
 #if defined(sun386)
 /* Sun386i run-time library bug in fputs(): only first line in s is written! */
@@ -280,7 +257,15 @@ typedef struct s_pattern_names
 #endif
 
 #if !defined(BIBCLEAN_INI)
-#define BIBCLEAN_INI		"BIBCLEANINI" /* environment variable */
+#define BIBCLEAN_INI		"BIBCLEANINI"	/* environment variable */
+#endif
+
+#if !defined(BIBCLEAN_ISBN)
+#define BIBCLEAN_ISBN		"BIBCLEANISBN"	/* environment variable */
+#endif
+
+#if !defined(BIBCLEAN_KEY)
+#define BIBCLEAN_KEY		"BIBCLEANKEY"	/* environment variable */
 #endif
 
 #define EMPTY_STRING(s)	(s[0] = (char)'\0', s)
@@ -345,8 +330,6 @@ int field_indentation = FIELD_INDENTATION;
 
 #include "token.h"
 
-#define NOOP			/* dummy statement */
-
 #if !defined(SCREEN_LINES)
 #if OS_PCDOS
 #define SCREEN_LINES	25	/* set 0 to disable pausing in out_lines() */
@@ -365,12 +348,21 @@ int value_indentation = VALUE_INDENTATION;
 /* Operating system-specific customizations. */
 
 #if OS_UNIX
-#if !defined(INITFILE)
-#define INITFILE	".bibcleanrc"
+
+#if !defined(INITDIR)
+#define INITDIR		("/usr/local/share/" PACKAGE_NAME "/" PACKAGE_VERSION)
 #endif
 
-#if !defined(SYSPATH)
-#define SYSPATH		"PATH"
+#if !defined(INITFILE)
+#define INITFILE	(PACKAGE_NAME ".ini")
+#endif
+
+#if !defined(ISBNFILE)
+#define ISBNFILE	(PACKAGE_NAME ".isbn")
+#endif
+
+#if !defined(KEYWORDFILE)
+#define KEYWORDFILE	(PACKAGE_NAME ".key")
 #endif
 
 #if !defined(USERPATH)
@@ -381,11 +373,7 @@ int value_indentation = VALUE_INDENTATION;
 
 #if OS_VAXVMS
 #if !defined(INITFILE)
-#define INITFILE	"bibclean.ini"
-#endif
-
-#if !defined(SYSPATH)
-#define SYSPATH		"SYS$SYSTEM"
+#define INITFILE	(PACKAGE_NAME ".ini")
 #endif
 
 #if !defined(USERPATH)
@@ -396,11 +384,15 @@ int value_indentation = VALUE_INDENTATION;
 
 /* For any that are undefined, default to values suitable for OS_PCDOS. */
 #if !defined(INITFILE)
-#define INITFILE	"bibclean.ini"
+#define INITFILE	(PACKAGE_NAME ".ini")
 #endif
 
-#if !defined(SYSPATH)
-#define SYSPATH		"PATH"
+#if !defined(ISBNFILE)
+#define ISBNFILE	(PACKAGE_NAME ".isb")
+#endif
+
+#if !defined(KEYWORDFILE)
+#define KEYWORDFILE	(PACKAGE_NAME ".key")
 #endif
 
 #if !defined(USERPATH)
@@ -409,7 +401,7 @@ int value_indentation = VALUE_INDENTATION;
 
 #include "toklst.h"
 
-const char *type_name[] =
+static const char *type_name[] =
 {				/* must be indexable by TOKEN_xxx */
     "UNKNOWN",
     "ABBREV",			/* alphabetical order, starting at 1 */
@@ -437,17 +429,17 @@ const char *type_name[] =
 limitations on external name lengths in ISO/ANSI Standard C.  Please
 keep them in ALPHABETICAL order, ignoring letter case. */
 
-void		do_initfile ARGS((const char *pathlist_,const char *name_));
+void		do_initfile ARGS((/*@null@*/ const char *pathlist_, /*@null@*/ const char *name_));
 YESorNO		apply_function ARGS((const char *option_,
 		    OPTION_FUNCTION_ENTRY table_[]));
 void		check_inodes ARGS((void));
 void		error ARGS((const char *msg_));
-void		fatal ARGS((const char *msg_));
+/*@noreturn@*/ void	fatal ARGS((const char *msg_));
 void		free_pattern_table_entries ARGS((PATTERN_TABLE *pt_));
 int		get_char ARGS((void));
 int		get_linebreak ARGS((void));
 int		get_next_non_blank ARGS((void));
-int		isidchar ARGS((int c_));
+bool		is_idchar ARGS((int c_));
 int		main ARGS((int argc_, char *argv_[]));
 void		out_at ARGS((void));
 void		out_c ARGS((int c_));
@@ -459,17 +451,21 @@ void		out_s ARGS((const char *s_));
 void		out_spaces ARGS((int n_));
 void		out_string ARGS((token_t type_, const char *token_));
 void		out_token ARGS((token_t type_, const char *token_));
-void		out_verbatim ARGS((const char *token_));
 void		out_with_error ARGS((const char *s_,const char *msg_));
 void		out_with_parbreak_error ARGS((char *s_));
 void		put_back ARGS((int c_));
-FILE		*tfopen ARGS((const char *filename_, const char *mode_));
+/*@null@*/ FILE	*tfopen ARGS((const char *filename_, const char *mode_));
 void		warning ARGS((const char *msg_));
 
 extern void	do_args ARGS((int argc_, char *argv_[]));
 extern void	do_files ARGS((int argc_, char *argv_[]));
+extern void	do_keyword_file ARGS((/*@null@*/ const char *pathlist_, /*@null@*/ const char *name_));
 extern void	do_other ARGS((void));
 extern void	do_preargs ARGS((int argc_, char *argv_[]));
+extern void	do_print_keyword_table ARGS((void));
+extern void	finish ARGS((int status));
+extern void	free_keyword_table ARGS((void));
+extern void	keyword_initialize ARGS((void));
 
 static long	bcolumn ARGS((void));
 static void	bdelc ARGS((void));
@@ -494,9 +490,10 @@ static void	out_number ARGS((long n_));
 static void	out_position ARGS((FILE *fpout_,const char *msg_,
 		    IO_PAIR *the_location_));
 static void	out_status ARGS((FILE *fpout_, const char *prefix_));
+static void	out_verbatim ARGS((const char *token_));
 static void	resync ARGS((void));
 extern char	*Strdup ARGS((const char *s_));
-static int	word_length ARGS((const char *s_));
+static size_t	word_length ARGS((const char *s_));
 static void	wrap_line ARGS((void));
 
 
@@ -508,7 +505,9 @@ ignoring letter case. */
 YESorNO		align_equals = NO;		/* NO: left-adjust equals */
 int		at_level = 0;			/* @ nesting level */
 int		brace_level = 0;		/* curly brace nesting level */
-static int	buf_length = 0;			/* length of buf[] */
+YESorNO		brace_math = YES;		/* NO: leave mixed-case math text untouched */
+YESorNO		brace_protect = YES;		/* NO: leave mixed-case title words untouched */
+static size_t	buf_length = 0;			/* length of buf[] */
 static char	buf[MAX_BUFFER+1];		/* 1 extra slot for trailing NUL */
 YESorNO		check_values = YES;		/* NO: suppress value checks */
 int		close_char = EOF;		/* BibTeX entry closing; may */
@@ -522,9 +521,13 @@ YESorNO		discard_next_comma = NO; 	/* YES: deleting field/value */
 YESorNO		eofile = NO;			/* set to YES at end-of-file */
 static int	error_count = 0;		/* used to decide exit code */
 						/* normalizing names */
-YESorNO		fix_initials = YES;		/* reformat A.U. Thor?	*/
-YESorNO		fix_names = YES;		/* reformat Bach, P.D.Q? */
+YESorNO		fix_accents = NO;		/* repair accent bracing? */
+YESorNO		fix_braces = NO;		/* normalize bracing? */
+YESorNO		fix_degrees = NO;		/* normalize author academic degrees? */
 YESorNO		fix_font_changes = NO;		/* brace {\em E. Coli}? */
+YESorNO		fix_initials = YES;		/* reformat A.U. Thor?	*/
+YESorNO		fix_math = NO;			/* reformat math mode? */
+YESorNO		fix_names = YES;		/* reformat Bach, P.D.Q? */
 
 #if defined(DEBUG)
 static FILE	*fpdebug;			/* for debugging */
@@ -543,13 +546,15 @@ YESorNO		keep_preamble_spaces = NO; /* YES: keep spaces in @Preamble{} */
 YESorNO		keep_spaces = NO;	/* YES: keep spaces in values */
 YESorNO		keep_string_spaces = NO; /* YES: keep spaces in @String{} */
 
-long		max_width = MAX_WIDTH;
+long		max_width;
 
 int		non_white_chars = 0;		/* used to test for legal @ */
 static IO_PAIR	original_file;			/* used in error messages */
 YESorNO		parbreaks = YES;		/* NO: parbreaks forbidden */
 						/* in strings and entries */
 YESorNO		prettyprint = YES;		/* NO: do lexical analysis */
+YESorNO		print_ISBN_table = NO;		/* YES: print ISBN table */
+YESorNO		print_keyword_table = NO;	/* YES: print keyword table */
 YESorNO		print_patterns = NO;		/* YES: print value patterns */
 char		*program_name;			/* set to argv[0] */
 
@@ -622,7 +627,9 @@ OPTION_FUNCTION_ENTRY table[];
 #endif
 {	/* return YES if function matching option was invoked, otherwise NO */
     int k;			/* index into table[] */
-    size_t n = strlen(option);	/* all chars of option[] will be examined */
+    size_t n;
+
+    n = strlen(option);		/* all chars of option[] will be examined */
 
     for (k = 0; table[k].name != (const char*)NULL; ++k)
     {
@@ -683,7 +690,7 @@ bdelline(VOID)				/* delete last line in buf[] */
 static void
 bflush(VOID)				/* output all but last line in buf[] */
 {
-    int	k;
+    size_t k;
 
     /* Output all but the last line of buf[], unless there is only one
        line, in which case, output all of buf[].  We need to keep a
@@ -691,14 +698,14 @@ bflush(VOID)				/* output all but last line in buf[] */
 
     buf[buf_length] = '\0';		/* terminate buffer string */
     for (k = buf_length; (k > 0) && (buf[k-1] != '\n'); --k)
-	NOOP;				/* find preceding newline */
-    if ((k >= 0) && (buf[k] == '\n')) /* then found last line */
+	continue;			/* find preceding newline */
+    if (buf[k] == '\n')			/* then found last line */
     {
 	buf[k] = (char)'\0';		/* zap the newline */
 	(void)fputs(buf,stdout);	/* output up to the zapped newline */
 	(void)fputc('\n',stdout);	/* output the zapped newline */
-	buf_length = (int)strlen(&buf[k+1]);
-	(void)Memmove(buf,&buf[k+1],(size_t)buf_length);
+	buf_length = strlen(&buf[k+1]);
+	(void)Memmove(buf,&buf[k+1],buf_length);
 					/* move last line to start of buf[] */
 					/* NB: overlap requires Memmove(), */
 					/* NOT strcpy()! */
@@ -715,7 +722,7 @@ bflush(VOID)				/* output all but last line in buf[] */
 static int
 blastc(VOID)			/* return last character in buffer, or EOF */
 {
-    return (buf_length > 0) ? buf[buf_length-1] : EOF;
+    return (buf_length > 0) ? (int)buf[buf_length-1] : EOF;
 }
 
 
@@ -728,9 +735,10 @@ bpeekc(offset)
 int offset;
 #endif
 {
-    int k = buf_length - 1 + offset;
+    int k;
 
-    return ((0 <= k) && (k < buf_length)) ? buf[k] : EOF;
+    k = (int)buf_length - 1 + offset;
+    return ((0 <= k) && (k < (int)buf_length)) ? (int)buf[k] : EOF;
 }
 
 
@@ -797,6 +805,7 @@ const char *msg;		/* default provided if this is NULL */
 }
 
 
+/*@noreturn@*/
 #if defined(HAVE_STDC)
 void				/* issue an error message and die */
 fatal(const char *msg)
@@ -811,6 +820,22 @@ const char *msg;
 }
 
 
+#if defined(HAVE_STDC)
+void
+finish(int status)
+#else /* K&R style */
+void
+finish(status)
+int status;
+#endif
+{
+    if (status == EXIT_SUCCESS)
+	free_tables();
+
+    exit(status);
+}
+
+
 #if defined(HAVE_STDC)
 static char *
 format(const char *msg)
@@ -829,7 +854,7 @@ const char *msg;
 #define ORIGINAL_MESSAGE (strncpy(newmsg,msg,MAX_TOKEN_SIZE), \
 	newmsg[MAX_TOKEN_SIZE-1] = (char)'\0', newmsg)
 
-    for (k = 0, n = 0; msg[k]; ++k)
+    for (k = 0, n = 0; msg[k] != '\0'; ++k)
     {
 	switch (msg[k])
 	{
@@ -900,13 +925,13 @@ MATCH_PATTERN *mp;
 {
     if (mp->pattern != (const char*)NULL)
     {					/* NB: (void*) cast fails with Sun C++ */
-        FREE(mp->pattern);
-        mp->pattern = (const char*)NULL;
+	FREE(mp->pattern);
+	mp->pattern = (const char*)NULL;
     }
     if (mp->message != (const char*)NULL)
     {
-        FREE(mp->message);
-        mp->message = (const char*)NULL;
+	FREE(mp->message);
+	mp->message = (const char*)NULL;
     }
 }
 
@@ -922,8 +947,8 @@ PATTERN_TABLE *pt;
 {
     if (pt != (PATTERN_TABLE*)NULL)
     {
-        free_pattern_table_entries(pt);
-        FREE(pt);
+	free_pattern_table_entries(pt);
+	FREE(pt);
     }
 }
 
@@ -937,14 +962,19 @@ free_pattern_table_entries(pt)
 PATTERN_TABLE *pt;
 #endif
 {
-    int k;
-
     if (pt != (PATTERN_TABLE*)NULL)
     {
+	int k;
+
 	for (k = 0; k < pt->current_size; ++k)
 	    free_match_pattern(&(pt->patterns[k]));
+
 	if (pt->patterns != (MATCH_PATTERN*)NULL)
+	{
 	    FREE(pt->patterns);
+	    pt->patterns = (MATCH_PATTERN*)NULL;
+	}
+
 	pt->current_size = 0;
 	pt->maximum_size = 0;
     }
@@ -963,6 +993,9 @@ free_tables(VOID)
 	pattern_names[k].name = (const char*)NULL;
 	pattern_names[k].table = (PATTERN_TABLE *)NULL;
     }
+
+    free_keyword_table();
+    free_ISBN_table();
 }
 
 
@@ -980,7 +1013,7 @@ get_char(VOID)			/* all input is read through this function */
 
     if (c == EOF)
 	eofile = YES;
-    else if (c == '\n')
+    else if (c == (int)'\n')
     {
 	the_file.input.line_number++;
 	the_file.input.last_column_position = the_file.input.column_position;
@@ -993,7 +1026,7 @@ get_char(VOID)			/* all input is read through this function */
 	the_file.input.column_position++;
 	non_white_chars++;
     }
-    else if (c == '\t')
+    else if (c == (int)'\t')
     {
 	the_file.input.last_column_position = the_file.input.column_position;
 	the_file.input.column_position =
@@ -1005,9 +1038,9 @@ get_char(VOID)			/* all input is read through this function */
 	the_file.input.column_position++;
     }
 
-    if (c == '{')
+    if (c == (int)'{')
 	brace_level++;
-    else if (c == '}')
+    else if (c == (int)'}')
 	brace_level--;
 
 #if defined(DEBUG)
@@ -1029,31 +1062,23 @@ get_linebreak(VOID)
     int c;
     int ns = 0;
 
-    while (((c = get_char()) != EOF) && Isspace(c))
+    while (((c = get_char(), c) != EOF) && Isspace(c))
     {
 	switch (c)
 	{
 	case '\n':
-#if 0
-	    return (((in_value == YES) && (keep_linebreaks == YES)) ?
-		LINEBREAK : c);
-#else
 	    return (LINEBREAK);
-#endif
+
 	case '\f':
-#if 0
-	    return (((in_value == YES) && (keep_linebreaks == YES)) ?
-		PARBREAK : c);
-#else
 	    return (PARBREAK);
-#endif
+
 	default:
 	    ns++;
 	    break;
 	}
     }
     put_back(c);
-    return ((ns > 0) ? ' ' : '\0');
+    return ((ns > 0) ? (int)' ' : (int)'\0');
 }
 
 
@@ -1061,8 +1086,6 @@ int
 get_next_non_blank(VOID)
 {
     int c;
-    int ff = 0;
-    int nl = 0;
 
     if (keep_spaces == YES)
 	c = get_char();
@@ -1072,27 +1095,37 @@ get_next_non_blank(VOID)
 	c = get_char();
     else
     {
+	int ff = 0;
+	int nl = 0;
+
 	is_parbreak = NO;
-	while (((c = get_char()) != EOF) && Isspace(c))
+
+	while (((c = get_char(), c) != EOF) && Isspace(c))
 	{
 	    switch (c)
 	    {
 	    case '\n':
 		if ((in_value == YES) && (keep_linebreaks == YES))
 		    return (LINEBREAK);
+
 		nl++;
 		break;
+
 	    case '\f':
 		if ((in_value == YES) && (keep_linebreaks == YES))
 		    return (c);
+
 		ff++;
 		break;
+
 	    default:
 		break;
 	    }
 	}
+
 	is_parbreak = ((nl > 1) || (ff > 0)) ? YES : NO;
     }
+
     return (c);
 }
 
@@ -1118,27 +1151,33 @@ init_tables(VOID)
 
 
 #if defined(HAVE_STDC)
-int
-isidchar(int c)
+bool
+is_idchar(int c)
 #else /* K&R style */
-int
-isidchar(c)
+bool
+is_idchar(c)
 int c;
 #endif
 {
-    /* See LaTeX User's Guide and Reference Manual, Section B.1.3, for the
-       rules of what characters can be used in a BibTeX word value.  Section
-       30 of BibTeX initializes id_class[] to match this, but curiously,
-       allows ASCII DELete (0x3f), as an identifier character.  This
-       irregularity has been reported to Oren Patashnik on
-       [06-Oct-1990].  We disallow it here.
-
-       The Scribe syntax is simpler: letters, digits, ., #, &, and %. */
+    /*
+    ** See LaTeX User's Guide and Reference Manual, Section B.1.3, for
+    ** the rules of what characters can be used in a BibTeX word
+    ** value.  Section 30 of BibTeX initializes id_class[] to match
+    ** this, but curiously, allows ASCII DELete (0x3f), as an
+    ** identifier character.  That irregularity was reported to Oren
+    ** Patashnik on [06-Oct-1990].  We disallow it here.
+    **
+    ** The Scribe syntax is simpler: letters, digits, ., #, &, and %.
+    */
 
     return ((Scribe == YES) ?
-	    (Isalnum(c) || (c == '.') || (c == '#') ||
-			   (c == '&') || (c == '%') ) :
-	    (Isgraph(c) && (strchr("\"#%'(),={}",c) == (char*)NULL)) );
+	    ((Isalnum(c) ||
+	      (c == (int)'.') ||
+	      (c == (int)'#') ||
+	      (c == (int)'&') ||
+	      (c == (int)'%') ) ? YES : NO) :
+	    ((Isgraph(c) &&
+	      !IN_SET("\"#%'(),={}", c)) ? YES : NO) );
 }
 
 
@@ -1153,6 +1192,10 @@ char *argv[];
 #endif
 {
     const char *initfile;
+    const char *ISBN_file;
+    const char *keyword_file;
+    const char *initdir;
+    static char alt_initdir[MAXPATHLEN + 1] = { '\0' };
 
 #if defined(vms)
     extern char **cmd_lin();
@@ -1160,7 +1203,49 @@ char *argv[];
     argv = cmd_lin( "", &argc, argv );
 #endif /* defined(vms) */
 
+#if defined(HAVE_SYS_PLEDGE_H)
+    /*
+    ** NB: The prototype for pledge() comes from <unistd.h>, but to
+    ** avoid possible conflicts with other O/Ses that might have
+    ** declare that name for another purpose, the configure script
+    ** actually checks for <sys/pledge.h>, which is more likely to be
+    ** the characteristic signature that OpenBSD-style pledge() is
+    ** supported.
+    **
+    ** The pledge() call, introduced in OpenBSD 5.9, and available in
+    ** later OpenBSD releases, enumerates classes of operations that
+    ** the program is permitted to do.  It can be issued more than
+    ** once, but only to further reduce privileges.  Its use here is
+    ** part of being a good software citizen on OpenBSD, and any other
+    ** operating systems that adopt the pledge mechanism for improving
+    ** system security.
+    **
+    ** The privileges required by bibclean are:
+    **
+    **     cpath	-- create new files
+    **
+    **     rpath 	-- read files and directories
+    **
+    **     stdio 	-- read and write stdio channels
+    **
+    **     tty		-- allow read/write/ioctl on /dev/tty
+    **     		   (needed for --help option)
+    **
+    **     wpath	-- write files
+    **
+    */
+
+    if (pledge("cpath rpath stdio tty wpath", NULL) != 0)
+    {
+	perror("pledge system call failed: perror says");
+	exit(EXIT_FAILURE);
+    }
+
+#endif
+
     initfile = GETDEFAULT(BIBCLEAN_INI,INITFILE);
+    ISBN_file = GETDEFAULT(BIBCLEAN_ISBN,ISBNFILE);
+    keyword_file = GETDEFAULT(BIBCLEAN_KEY,KEYWORDFILE);
 
     max_width = 0L;			/* reset later */
 
@@ -1168,10 +1253,66 @@ char *argv[];
 
     program_name = argv[0];
 
+    initdir = INITDIR;
+
+    if (alt_initdir[0] != '\0')		/* have value from previous call */
+	initdir = (const char *)&alt_initdir[0];
+    else if ( (program_name != (char *)NULL) && (program_name[0] != '\0') )
+    {
+	/*
+	** If INITDIR does not exist, then convert a path to the
+	** executable of, say, "/usr/local/bin/bibclean" to a
+	** directory name "/usr/local/bin/", then append to it a
+	** relative path to find the installation tree location,
+	** "/usr/local/share/bibclean/VERSION"
+	*/
+	struct stat stat_buf;
+
+	if ( (stat(initdir, &stat_buf) == 0) && (S_ISDIR(stat_buf.st_mode)) )
+	{	    /* success: initdir exists */
+	}
+	else		/* replace likely-relocated initdir at run-time  */
+	{
+	    char * p;
+
+	    (void)strncpy(alt_initdir, program_name, sizeof(alt_initdir));
+	    p = strrchr(alt_initdir, '/');
+
+	    if (p == (char *)NULL)
+		alt_initdir[0] = '\0';
+	    else
+	    {
+		*p = '\0';		/* terminate string at last directory */
+		p = strrchr(alt_initdir, '/');
+
+		if (p != (char *)NULL)
+		{
+		    p = strrchr(alt_initdir, '/');
+		    if (p != (char *)NULL)
+		    {
+			const char *q;
+
+			p[1] = '\0';	/* terminate string at second last directory */
+			q = ("share/" PACKAGE_NAME "/" PACKAGE_VERSION "/");
+			(void)strncat(alt_initdir, q, sizeof(alt_initdir) - strlen(q));
+			initdir = &alt_initdir[0];
+
+			if ( (stat(initdir, &stat_buf) == 0) && (S_ISDIR(stat_buf.st_mode)) )
+			{
+			    /* then relocated directory exists*/
+			}
+			else
+			    (void)fprintf(stderr, "WARNING: relocated %s installation directory does not exist\n", initdir);
+		    }
+		}
+	    }
+	}
+    }
+
     check_inodes();
 
 #if defined(DEBUG)
-    fpdebug = tfopen("bibclean.dbg", "w");
+    fpdebug = tfopen((PACKAGE_NAME ".dbg"), "w");
 #endif /* defined(DEBUG) */
 
     the_file.input.filename = "";
@@ -1182,25 +1323,43 @@ char *argv[];
     do_preargs(argc,argv);/* some args must be handled BEFORE initializations */
 
     if (read_initialization_files == YES)
-	do_initfile(getenv(SYSPATH),initfile);
+	do_initfile(initdir,initfile);
 
     if (read_initialization_files == YES)
 	do_initfile(getenv(USERPATH),initfile);
 
+    ISBN_initialize();
+    keyword_initialize();
+
+    if (read_initialization_files == YES)
+    {
+	do_ISBN_file(initdir,ISBN_file);
+	do_ISBN_file(getenv(USERPATH),ISBN_file);
+
+	do_keyword_file(initdir,keyword_file);
+	do_keyword_file(getenv(USERPATH),keyword_file);
+    }
+
     do_args(argc,argv);
 
+    if (print_ISBN_table == YES)
+    {
+	do_print_ISBN_table();
+	finish(EXIT_SUCCESS);
+    }
+
+    if (print_keyword_table == YES)
+    {
+	do_print_keyword_table();
+	finish(EXIT_SUCCESS);
+    }
+
     if (max_width == 0L)		/* set default value */
 	max_width = (prettyprint == YES) ? MAX_WIDTH : LONG_MAX;
 
     do_files(argc,argv);
 
-    free_tables();
-
-#if OS_VAXVMS
-    exit (error_count ? EXIT_FAILURE : EXIT_SUCCESS);
-#endif /* OS_VAXVMS */
-
-    return (error_count ? EXIT_FAILURE : EXIT_SUCCESS);
+    finish((error_count > 0) ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
 
@@ -1292,12 +1451,12 @@ int c;				/* and output buffer if c == EOF */
 
 	/* If the buffer ends with an inline comment, then we just
 	   deleted the newline that terminates it, so we must put it back */
-	for (offset = 0; ((c = bpeekc(offset)) != EOF) && (c != '\n');
+	for (offset = 0; ((c = bpeekc(offset)) != EOF) && (c != (int)'\n');
 	     offset--)
 	{
 	    if (c == BIBTEX_COMMENT_PREFIX)
 	    {
-		bputc('\n');
+		bputc((int)'\n');
 		break;
 	    }
 	}
@@ -1314,20 +1473,20 @@ int c;				/* and output buffer if c == EOF */
 #endif
 
     case '\n':
-	for (c = blastc(); (c == ' ') || (c == '\t'); c = blastc())
+	for (c = blastc(); (c == (int)' ') || (c == (int)'\t'); c = blastc())
 	    bdelc();			/* discard trailing horizontal space */
-	bputc('\n');
+	bputc((int)'\n');
 	break;
 
     default:		/* output ordinary character, with possible line wrap */
-	if ((prettyprint == NO) && (c != '\n'))
+	if ((prettyprint == NO) && (c != (int)'\n'))
 	{				/* need to line wrap? */
 	    if (bcolumn() > (max_width - 2))
 	    {				/* yes, output backslash-newline pair */
 		the_file.input.last_column_position =
 		    the_file.input.column_position;
-		bputc('\\');
-		bputc('\n');
+		bputc((int)'\\');
+		bputc((int)'\n');
 	    }
 	}
 	bputc(c);
@@ -1393,21 +1552,22 @@ YESorNO pausing;
     int k;
 
 #if (SCREEN_LINES > 0)
-    int lines_on_screen;
-    int nlines;
-
-    if (pausing == YES)
+    if ((pausing == YES) && (screen_lines > 0))
     {
+	int lines_on_screen;
+	int nlines;
+
 	kbopen();
 	for (nlines = 0; lines[nlines] != (const char*)NULL; ++nlines)
-	    NOOP;				/* count number of lines */
+	    continue;				/* count number of lines */
 
 	for (k = 0, lines_on_screen = 0; ; )
 	{
 	    if (lines[k] != (const char*)NULL)
 	    {
 		(void)fputs(lines[k], fpout);
-		if (strchr(lines[k],'\n') != (char*)NULL)
+
+		if (IN_SET(lines[k], '\n'))
 		    lines_on_screen++;	/* some lines[k] are only partial */
 	    }
 
@@ -1416,6 +1576,15 @@ YESorNO pausing;
 	    {					/* pause for user action */
 		lines_on_screen = 0;
 		screen_lines = get_screen_lines(); /* maybe window resize? */
+		if (screen_lines == 0)
+		{
+		    k++;
+		    if (k < nlines)
+			continue;
+		    else
+			break;			/* here's the loop exit */
+		}
+
 		k = do_more(fpout,k,screen_lines - 2,lines);
 		if (k == EOF)
 		    break;			/* here's the loop exit */
@@ -1511,7 +1680,7 @@ const char *s;
        that we can automatically account for the comma that will be
        supplied after the string. */
 
-    for (; *s; ++s)
+    for (; (*s != '\0'); ++s)
     {
 	switch (*s)
 	{
@@ -1521,21 +1690,21 @@ const char *s;
 		    > max_width)
 		wrap_line();
 	    else
-		out_c((unsigned char)*s);
+		out_c((int)(unsigned char)*s);
 	    break;
 
 	case PARBREAK:		/* Possible after first character only if */
 				/* keep_parbreaks == YES.  Cannot use */
 				/* wrap_line(), because that is infinitely */
 				/* recursive to this statement! */
-	    out_c((unsigned char)'\n');
-	    out_c((unsigned char)'\n'); /* two newlines marks a parbreak */
+	    out_c((int)(unsigned char)'\n');
+	    out_c((int)(unsigned char)'\n'); /* two newlines marks a parbreak */
 	    if (keep_spaces == NO)
 		out_spaces(VALUE_INDENTATION); /* supply leading indentation */
 	    break;
 
 	case LINEBREAK:
-	    out_c((unsigned char)'\n'); /* change LINEBREAK to newline */
+	    out_c((int)(unsigned char)'\n'); /* change LINEBREAK to newline */
 	    if (keep_spaces == NO)
 	    {
 		out_spaces(VALUE_INDENTATION); /* supply leading indentation */
@@ -1544,29 +1713,8 @@ const char *s;
 	    }
 	    break;
 
-#if 0
-	/* It appears that wrapping at these punctuation characters is a bad
-	   idea, because it can introduce line breaks in strings which should
-	   not be broken, such as file names, and WWW URL fields. */
-	case '!':		/* may wrap after certain punctuation */
-	case '&':
-	case '+':
-	case ',':
-	case '.':
-	case ':':
-	case ';':
-	case '=':
-	case '?':
-	    out_c((unsigned char)*s);
-	    if ((wrapping == YES) &&
-		(the_file.output.column_position + word_length(s+1))
-		    > max_width)
-		wrap_line();
-	    break;
-#endif
-
 	default:		/* everything else is output verbatim */
-	    out_c((unsigned char)*s);
+	    out_c((int)(unsigned char)*s);
 	}
     }
 }
@@ -1584,7 +1732,7 @@ int n;
     if (prettyprint == YES)
     {
 	for (; n > 0; --n)
-	    out_c(' ');
+	    out_c((int)' ');
     }
     /* If we are not prettyprinting, but lexically analyzing, we */
     /* cannot use n as a reliable count of spaces, because it is */
@@ -1666,53 +1814,53 @@ const char *token;
 	last_line_number = token_start.input.line_number;
     }
     out_number((long)type);
-    out_c('\t');
+    out_c((int)'\t');
     out_s(type_name[(int)type]);
-    out_c('\t');
-    out_c('"');
-    for (; *token; ++token)
+    out_c((int)'\t');
+    out_c((int)'"');
+    for (; (*token != '\0'); ++token)
     {
 	switch (*token)
 	{
 	case '"':
 	case '\\':
-	    out_c('\\');
-	    out_c(*token);
+	    out_c((int)'\\');
+	    out_c((int)*token);
 	    break;
 	case '\b':
-	    out_c('\\');
-	    out_c('b');
+	    out_c((int)'\\');
+	    out_c((int)'b');
 	    break;
 	case '\f':
-	    out_c('\\');
-	    out_c('f');
+	    out_c((int)'\\');
+	    out_c((int)'f');
 	    break;
 	case LINEBREAK:
 	case '\n':
-	    out_c('\\');
-	    out_c('n');
+	    out_c((int)'\\');
+	    out_c((int)'n');
 	    break;
 	case '\r':
-	    out_c('\\');
-	    out_c('r');
+	    out_c((int)'\\');
+	    out_c((int)'r');
 	    break;
 	case '\t':
-	    out_c('\\');
-	    out_c('t');
+	    out_c((int)'\\');
+	    out_c((int)'t');
 	    break;
 	case '\v':
-	    out_c('\\');
-	    out_c('v');
+	    out_c((int)'\\');
+	    out_c((int)'v');
 	    break;
 	case PARBREAK:		/* two newlines mark a parbreak */
-	    out_c('\\');
-	    out_c('n');
-	    out_c('\\');
-	    out_c('n');
+	    out_c((int)'\\');
+	    out_c((int)'n');
+	    out_c((int)'\\');
+	    out_c((int)'n');
 	    break;
 	default:
 	    if (Isprint(*token))
-		out_c((unsigned char)*token);
+		out_c((int)(unsigned char)*token);
 	    else
 	    {
 		(void)sprintf(octal,"\\%03o",BYTE_VAL(*token));
@@ -1721,33 +1869,33 @@ const char *token;
 	    break;
 	}
     }
-    out_c('"');
-    out_c('\n');
+    out_c((int)'"');
+    out_c((int)'\n');
 }
 
 
 #if defined(HAVE_STDC)
-void
+static void
 out_verbatim(const char *token)
 #else /* K&R style */
-void
+static void
 out_verbatim(token)
 const char *token;
 #endif
 {
-    for (; *token; ++token)
+    for (; (*token != '\0'); ++token)
     {
 	switch (*token)
 	{
 	case LINEBREAK:
-	    out_c((unsigned char)'\n');
+	    out_c((int)(unsigned char)'\n');
 	    break;
 	case PARBREAK:
-	    out_c((unsigned char)'\n');
-	    out_c((unsigned char)'\n');
+	    out_c((int)(unsigned char)'\n');
+	    out_c((int)(unsigned char)'\n');
 	    break;
 	default:
-	    out_c((unsigned char)*token);
+	    out_c((int)(unsigned char)*token);
 	    break;
 	}
     }
@@ -1808,18 +1956,18 @@ int c;
 
     if (c == EOF)
 	eofile = NO;
-    else if (c == '\n')
+    else if (c == (int)'\n')
     {
 	the_file.input.column_position = the_file.input.last_column_position;
 	the_file.input.line_number--;
     }
-    else if (c == '\t')
+    else if (c == (int)'\t')
 	the_file.input.column_position = the_file.input.last_column_position;
     else
 	the_file.input.column_position--;
-    if (c == '{')
+    if (c == (int)'{')
 	brace_level--;
-    else if (c == '}')
+    else if (c == (int)'}')
 	brace_level++;
 }
 
@@ -1861,9 +2009,9 @@ const char *s2;
 #endif
 {
 
-#define TOUPPER(c) (Islower(c) ? toupper(c) : (c))
+#define TOUPPER(c) (Islower(c) ? toupper((int)(c)) : (int)(c))
 
-    while ((*s1) && (TOUPPER(*s1) == TOUPPER(*s2)))
+    while ((*s1 != '\0') && (TOUPPER(*s1) == TOUPPER(*s2)))
     {
 	s1++;
 	s2++;
@@ -1894,10 +2042,10 @@ size_t		n;
       end-of-string, whichever comes first.
     *******************************************************************/
 
-    for (; (n > 0) && *s1 && *s2; ++s1, ++s2, --n)
+    for (; (n > 0) && (*s1 != '\0') && (*s2 != '\0'); ++s1, ++s2, --n)
     {
-	c1 = 0xff & (int)(Islower(*s1) ? (int)*s1 : tolower(*s1));
-	c2 = 0xff & (int)(Islower(*s2) ? (int)*s2 : tolower(*s2));
+	c1 = 0xff & (int)(Islower(*s1) ? (int)*s1 : tolower((int)(*s1)));
+	c2 = 0xff & (int)(Islower(*s2) ? (int)*s2 : tolower((int)(*s2)));
 	if (c1 < c2)
 	    return (-1);
 	else if (c1 > c2)
@@ -1913,9 +2061,11 @@ size_t		n;
 
 
 #if defined(HAVE_STDC)
+/*@null@*/
 FILE*
 tfopen(const char *filename, const char *mode) /* traced file opening */
 #else /* K&R style */
+/*@null@*/
 FILE*
 tfopen(filename,mode)
 const char *filename;
@@ -1926,7 +2076,7 @@ const char *mode;
 
     fp = FOPEN(filename,mode);
     if (trace_file_opening == YES)
-	(void)fprintf(stdlog,"%s open file [%s]%s\n",
+	(void)fprintf(stdlog,"%s open file   [%s]%s\n",	/* NB: lineup with lookup in fndfil.c:file_is_readable() */
 	    WARNING_PREFIX, filename, (fp == (FILE*)NULL) ? ": FAILED" : "");
     return (fp);
 }
@@ -1949,10 +2099,12 @@ const char *msg;
 	   start a new line if stdlog and stdout are the same file. */
 
 	original_file = the_file;	/* save for error messages */
-	(void)fprintf(stdlog,"%s%s \"%s\", line %ld: %s.\n",
+
+	(void)fprintf(stdlog,"%s%s %s:%ld:%s.\n",
 	    (stdlog_on_stdout == YES) ? "\n" : "",
 	    WARNING_PREFIX, the_file.input.filename,
 	    the_value.input.line_number, format(msg));
+
 	out_status(stdlog, WARNING_PREFIX);
 	(void)fflush(stdlog);
     }
@@ -1960,22 +2112,22 @@ const char *msg;
 
 
 #if defined(HAVE_STDC)
-static int
+static size_t
 word_length(const char *s)	/* return length of leading non-blank prefix */
 #else /* K&R style */
-static int
+static size_t
 word_length(s)			/* return length of leading non-blank prefix */
 const char *s;
 #endif
 {
     size_t n;
 
-    for (n = 0; s[n]; ++n)
+    for (n = 0; (s[n] != '\0'); ++n)
     {
 	if (Isspace(s[n]))
 	    break;
     }
-    return ((int)((s[n] == '\0') ? n + 1 : n));
+    return ((s[n] == '\0') ? n + 1 : n);
 				/* at end of string, return one more than */
 				/* true length to simplify line wrapping */
 }
