@@ -2,39 +2,14 @@
 #include "xctype.h"
 #include "xstat.h"
 #include "xstring.h"
+#include "xstdlib.h"
 
-RCSID("$Id: chek.c,v 1.3 1996/05/03 20:11:39 beebe Exp beebe $")
+RCSID("$Id: chek.c,v 1.10 2017/06/09 14:47:20 beebe Exp beebe $")
 
 #include "yesorno.h"
 #include "match.h"			/* must come AFTER yesorno.h */
 #include "token.h"
-
-typedef struct s_parse_data
-{
-    YESorNO	(*is_name_char) ARGS((int c_, size_t n_));
-    const char	*s;		/* pointer to next char in list */
-    const char	*token;		/* pointer to token in list */
-    int		token_length;	/* number of token characters (== (s - token)) */
-} parse_data;
-
-typedef struct s_pattern_table
-{
-    MATCH_PATTERN *patterns;
-    int current_size;
-    int maximum_size;
-} PATTERN_TABLE;
-
-typedef struct s_pattern_names
-{
-    const char *name;
-    PATTERN_TABLE *table;
-} PATTERN_NAMES;
-
-typedef struct s_name_pair
-{
-    const char *old_name;
-    const char *new_name;
-} NAME_PAIR;
+#include "typedefs.h"			/* must come AFTER match.h */
 
 #if defined(HAVE_PATTERNS)
 #define PATTERN_MATCHES(string,pattern) (match_pattern(string,pattern) == YES)
@@ -49,7 +24,15 @@ typedef struct s_name_pair
 #define	PT_VOLUME	4		/* index in pattern_names[] */
 #define	PT_YEAR		5		/* index in pattern_names[] */
 
-#define STD_MAX_TOKEN	((size_t)1000)	/* Standard BibTeX limit */
+#if !defined(STD_MAX_TOKEN)
+/* This was 1000 in original BibTeX, then changed to 5000 for TeX Live */
+/* builds up to 2011, and changed again for 2012--2019 releases to 20000 */
+/* See glob_str_size in : $prefix/texlive/YYYY/texmf-dist/web2c/texmf.cnf */
+/* #define STD_MAX_TOKEN	((size_t)1000)	*/ /* Standard BibTeX limit */
+/* #define STD_MAX_TOKEN	((size_t)20000) */ /* TeX Live 2012--2018 limit */
+#define STD_MAX_TOKEN	((size_t)200000)	/* TeX Live 2019--2021 limit */
+
+#endif /* !defined(STD_MAX_TOKEN) */
 
 #define UNKNOWN_CODEN	"??????"
 #define MAX_CODEN	(sizeof(UNKNOWN_CODEN)-1)
@@ -57,6 +40,9 @@ typedef struct s_name_pair
 #define UNKNOWN_ISBN	"??????????"
 #define MAX_ISBN	(sizeof(UNKNOWN_ISBN)-1)
 
+#define UNKNOWN_ISBN_13	"?????????????"
+#define MAX_ISBN_13	(sizeof(UNKNOWN_ISBN_13)-1)
+
 #define UNKNOWN_ISSN	"????????"
 #define MAX_ISSN	(sizeof(UNKNOWN_ISSN)-1)
 
@@ -71,10 +57,12 @@ extern FILE	*stdlog;		/* usually stderr */
 extern YESorNO	stdlog_on_stdout;	/* NO for separate files */
 
 extern void	error ARGS((const char *msg_));
-extern void	ISBN_hyphenate ARGS((char *s_,char *t_,size_t maxs_));
+extern void	ISBN_hyphenate ARGS((/*@out@*/ char *s_, /*@out@*/ char *t_, size_t maxs_));
+extern void	ISBN_13_hyphenate ARGS((/*@out@*/ char *s_, /*@out@*/ char *t_, size_t maxs_));
 extern void	warning ARGS((const char *msg_));
 
 void		check_chapter ARGS((void));
+void		check_DOI ARGS((void));
 void		check_CODEN ARGS((void));
 void		check_ISBN ARGS((void));
 void		check_ISSN ARGS((void));
@@ -85,31 +73,41 @@ void		check_month ARGS((void));
 void		check_number ARGS((void));
 void		check_other ARGS((void));
 void		check_pages ARGS((void));
-YESorNO		check_patterns ARGS((PATTERN_TABLE *pt_,const char *value_));
+void		check_URL ARGS((void));
 void		check_volume ARGS((void));
 void		check_year ARGS((void));
 
 static void	bad_CODEN ARGS((char CODEN_[6]));
 static void	bad_ISBN ARGS((char ISBN_[11]));
+static void	bad_ISBN_13 ARGS((char ISBN_13_[14]));
 static void	bad_ISSN ARGS((char ISSN_[9]));
+static YESorNO	check_patterns ARGS((PATTERN_TABLE *pt_,const char *value_));
 static int	CODEN_character_value ARGS((int c_));
 static size_t	copy_element ARGS((char *target_, size_t nt_, const char *source_, size_t ns_));
 static void	incomplete_CODEN ARGS((char CODEN_[6]));
 static YESorNO	is_CODEN_char ARGS((int c_, size_t n_));
+static YESorNO	is_DOI_char ARGS((int c_, size_t n_));
 static YESorNO	is_ISBN_char ARGS((int c_, size_t n_));
+static YESorNO	is_ISBN_13_char ARGS((int c_, size_t n_));
 static YESorNO	is_ISSN_char ARGS((int c_, size_t n_));
+static YESorNO	is_URL_char ARGS((int c_, size_t n_));
 static void	parse_list ARGS((const char *s,
 				 YESorNO (*is_name_char_) ARGS((int c_, size_t n_)),
 				 void (*validate_) ARGS((const char *CODEN_, size_t n_))));
-static void	parse_element ARGS((parse_data *pd_));
-static void	parse_separator ARGS((parse_data *pd_));
+static void	parse_element ARGS((/*@out@*/ parse_data *pd_));
+static void	parse_separator ARGS((/*@out@*/ parse_data *pd_));
 static void	validate_CODEN ARGS((const char *CODEN_, size_t n_));
+static void	validate_DOI ARGS((const char *CODEN_, size_t n_));
 static void	validate_ISBN ARGS((const char *ISBN_, size_t n_));
+static void	validate_ISBN_13 ARGS((const char *ISBN_, size_t n_));
 static void	validate_ISSN ARGS((const char *ISSN_, size_t n_));
+static void	validate_URL ARGS((const char *CODEN_, size_t n_));
 static void	unexpected ARGS((void));
 
-#define ISBN_DIGIT_VALUE(c)	((((c) == 'X') || ((c) == 'x')) ? 10 : \
-					((c) - '0'))
+#define elementsof(v)		(sizeof(v) / sizeof(v[0]))
+
+#define ISBN_DIGIT_VALUE(c)	((((int)(c) == (int)'X') || ((int)(c) == (int)'x')) ? 10 : \
+					((int)(c) - (int)'0'))
 				/* correct only if digits are valid; */
 				/* the code below ensures that */
 
@@ -125,7 +123,7 @@ bad_CODEN(CODEN)
 char CODEN[7];
 #endif
 {
-    static char fmt[] =
+    static const char fmt[] =
 	"Invalid checksum for CODEN %c%c%c%c%c%c in ``%%f = %%v''";
     char msg[sizeof(fmt)];
 
@@ -148,7 +146,7 @@ char ISBN[11];
 #endif
 {
 #define MAXISBN	(13+1)	/* space for correctly hyphenated ISBN, plus NUL */
-    static char fmt[] = "Invalid checksum for ISBN %s in ``%%f = %%v''";
+    static const char fmt[] = "Invalid checksum for ISBN %s in ``%%f = %%v''";
     char msg[sizeof(fmt)+MAXISBN-1-2];
     char s[MAXISBN];
     char t[MAXISBN];
@@ -166,6 +164,34 @@ char ISBN[11];
 }
 
 
+#if defined(HAVE_STDC)
+static void
+bad_ISBN_13(char ISBN_13[13 + 1])
+#else /* K&R style */
+static void
+bad_ISBN_13(ISBN_13)
+char ISBN_13[13 + 1];
+#endif
+{
+#define MAXISBN_13	(13 + 3 + 1)	/* space for correctly hyphenated ISBN_13, plus NUL */
+    static const char fmt[] = "Invalid checksum for ISBN_13 %s in ``%%f = %%v''";
+    char msg[sizeof(fmt) + MAXISBN_13 - 1 - 2];
+    char s[MAXISBN_13];
+    char t[MAXISBN_13];
+    size_t n;
+
+    (void)strcpy(s,UNKNOWN_ISBN_13);
+    n = strlen(&ISBN_13[1]);
+    (void)memcpy(s,&ISBN_13[1],(n > sizeof(s)) ? sizeof(s) : n);
+    s[13] = '\0';
+    ISBN_13_hyphenate(s,t,sizeof(s));
+
+    (void)sprintf(msg, fmt, s);
+    warning(msg);	/* used to be error(), but some books actually have */
+			/* invalid ISBN_13s */
+}
+
+
 #if defined(HAVE_STDC)
 static void
 bad_ISSN(char ISSN[9])
@@ -175,7 +201,7 @@ bad_ISSN(ISSN)
 char ISSN[9];
 #endif
 {
-    static char fmt[] =
+    static const char fmt[] =
 	"Invalid checksum for ISSN %c%c%c%c-%c%c%c%c in ``%%f = %%v''";
     char msg[sizeof(fmt)];
 
@@ -219,6 +245,18 @@ check_CODEN(VOID)
 }
 
 
+void
+check_DOI(VOID)
+{
+    parse_list(current_value, is_DOI_char, validate_DOI);
+
+    if ( IN_SET(current_value, ' ') ||
+	 IN_SET(current_value, ',') ||
+	 IN_SET(current_value, ';') )
+	warning("Unexpected space or list separator in DOI value ``%v''");
+}
+
+
 void
 check_inodes(VOID)
 {
@@ -266,6 +304,18 @@ check_ISBN(VOID)
 }
 
 
+void
+check_ISBN_13(VOID)
+{
+    char t[MAX_TOKEN_SIZE];
+
+    /* Supply correct hyphenation for all ISBN-13s */
+    ISBN_13_hyphenate(current_value, t, sizeof(t) / sizeof(t[0]));
+
+    parse_list(current_value, is_ISBN_13_char, validate_ISBN_13);
+}
+
+
 void
 check_ISSN(VOID)
 {
@@ -273,6 +323,16 @@ check_ISSN(VOID)
 }
 
 
+void
+check_ISSN_L(VOID)
+{
+    parse_list(current_value, is_ISSN_char, validate_ISSN);
+
+    if (strlen(current_value) != 11)	/* "1234-5689" */
+	warning("Unexpected ISSN-L field length in ``%v''");
+}
+
+
 #if defined(HAVE_STDC)
 YESorNO
 check_junior(const char *last_name)
@@ -297,7 +357,7 @@ const char *last_name;
     int k;			/* index into juniors[] */
     int n;			/* index into last_name[] */
 
-    for (n = 0, b_level = 0; last_name[n]; ++n)
+    for (n = 0, b_level = 0; last_name[n] != '\0'; ++n)
     {				/* check for "Smith, Jr" and "Smith Jr" and */
 	switch (last_name[n])	/* convert to "{Smith, Jr}" and "{Smith Jr}" */
 	{
@@ -371,11 +431,14 @@ size_t n;
 void
 check_month(VOID)
 {
-    int m;			/* month index */
-    size_t n = strlen(current_value);
+    size_t n;
+
+    n = strlen(current_value);
 
     if (n == 3)			/* check for match against standard abbrevs */
     {
+	int m;			/* month index */
+
 	for (m = 0; month_pair[m].old_name != (const char*)NULL; ++m)
 	{
 	    if (stricmp(month_pair[m].new_name,current_value) == 0)
@@ -519,10 +582,10 @@ check_pages(VOID)
 #if (defined(HAVE_PATTERNS) || defined(HAVE_REGEXP) || defined(HAVE_RECOMP))
 
 #if defined(HAVE_STDC)
-YESorNO
+static YESorNO
 check_patterns(PATTERN_TABLE* pt,const char *value)
 #else /* K&R style */
-YESorNO
+static YESorNO
 check_patterns(pt,value)
 PATTERN_TABLE* pt;
 const char *value;
@@ -555,6 +618,13 @@ const char *value;
 	   defined(HAVE_RECOMP)) */
 
 
+void
+check_URL(VOID)
+{
+    parse_list(current_value, is_URL_char, validate_URL);
+}
+
+
 void
 check_volume(VOID)
 {
@@ -573,7 +643,7 @@ check_volume(VOID)
 		|| (current_value[k] == '-')
 		|| (current_value[k] == '/')
 		|| (current_value[k] == '.')
-		|| isspace(current_value[k])
+		|| Isspace(current_value[k])
 		|| (current_value[k] == '{')
 		|| (current_value[k] == '}') ))
 	{
@@ -616,7 +686,7 @@ check_year(VOID)
     unexpected();
 #endif /* defined(HAVE_PATTERNS) */
 
-    for (p = current_value; *p ; ) /* now validate all digit strings */
+    for (p = current_value; (*p != '\0') ; ) /* now validate all digit strings */
     {
 	if (Isdigit(*p))	/* then have digit string */
 	{			/* now make sure year is `reasonable' */
@@ -640,13 +710,13 @@ CODEN_character_value(c)
 int c;
 #endif
 {
-    if (('a' <= c) && (c <= 'z'))
-	return ((c - 'a' + 1));
-    else if (('A' <= c) && (c <= 'Z'))
-	return ((c - 'A' + 1));
-    else if (('1' <= c) && (c <= '9'))
-	return ((c - '1' + 27));
-    else if (c == '0')
+    if (((int)'a' <= c) && (c <= (int)'z'))
+	return ((c - (int)'a' + 1));
+    else if (((int)'A' <= c) && (c <= (int)'Z'))
+	return ((c - (int)'A' + 1));
+    else if (((int)'1' <= c) && (c <= (int)'9'))
+	return ((c - (int)'1' + 27));
+    else if (c == (int)'0')
 	return (36);
     else
 	return (-1);
@@ -658,7 +728,11 @@ static size_t
 copy_element(char *target, size_t nt, const char *source, size_t ns)
 #else /* K&R style */
 static size_t
-copy_element(char *target, size_t nt, const char *source, size_t ns)
+copy_element(target, nt, source, ns)
+char *target;
+size_t nt;
+const char *source;
+size_t ns;
 #endif
 {    /* Copy source[] into target[], excluding spaces and hyphens, and add a */
      /* trailing NUL.  Return the number of characters left in source[], */
@@ -668,13 +742,13 @@ copy_element(char *target, size_t nt, const char *source, size_t ns)
 
     for (ks = 0, kt = 0; (ks < ns) && (kt < (nt - 1)); ++ks)
     {
-	if (!((source[ks] == '-') || isspace(source[ks])))
+	if (!((source[ks] == '-') || Isspace(source[ks])))
 	    target[kt++] = source[ks];
     }
     target[kt] = '\0';
 
-    for ( ; (source[ks] == '-') || isspace(source[ks]); ++ks)
-	/* NO-OP */;		 /* skip trailing space and hyphens */
+    for ( ; (source[ks] == '-') || Isspace(source[ks]); ++ks)
+	continue;		 /* skip trailing space and hyphens */
 
     return (size_t)(ns - ks);
 }
@@ -689,7 +763,7 @@ incomplete_CODEN(CODEN)
 char CODEN[7];
 #endif
 {
-    static char fmt[] =
+    static const char fmt[] =
 	"Incomplete CODEN %c%c%c%c%c should be %c%c%c%c%c%c in ``%%f = %%v''";
     char msg[sizeof(fmt)];
 
@@ -720,14 +794,14 @@ size_t n;
 	n_significant = 0;
 
     /* embedded hyphens are accepted, but are not significant */
-    if ((n_significant > 0) && (c == '-'))
+    if ((n_significant > 0) && (c == (int)'-'))
 	return (YES);
-    else if ((n_significant < 5) && isalpha(c))
+    else if ((n_significant < 5) && Isalpha(c))
     {
 	n_significant++;
 	return (YES);
     }
-    else if ((n_significant >= 5) && isalnum(c)) /* sixth char can be a digit */
+    else if ((n_significant >= 5) && Isalnum(c)) /* sixth char can be a digit */
     {
 	n_significant++;
 	return (YES);
@@ -737,6 +811,20 @@ size_t n;
 }
 
 
+#if defined(HAVE_STDC)
+static YESorNO
+is_DOI_char(int c, size_t n)
+#else /* K&R style */
+static YESorNO
+is_DOI_char(c,n)
+int c;
+size_t n;
+#endif
+{
+    return (Isprint(c) ? YES : NO);    /* DOIs match any printable string */
+}
+
+
 #if defined(HAVE_STDC)
 static YESorNO
 is_ISBN_char(int c, size_t n)
@@ -752,21 +840,21 @@ size_t n;
 
     /* ISBNs match
 	[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9][- ]*
-        [0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9xX]
+	[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9xX]
     */
 
     if (n == 0)				/* start new ISBN */
 	n_significant = 0;
 
     /* embedded hyphens and space are accepted, but are not significant */
-    if ((n_significant > 0) && ((c == '-') || isspace(c)))
+    if ((n_significant > 0) && ((c == (int)'-') || Isspace(c)))
 	return (YES);
-    else if ((n_significant < 9) && isdigit(c))
+    else if ((n_significant < 9) && Isdigit(c))
     {
 	n_significant++;
 	return (YES);
     }
-    else if ((n_significant >= 9) && (isdigit(c) || (c == 'X') || (c == 'x')))
+    else if ((n_significant >= 9) && (Isdigit(c) || (c == (int)'X') || (c == (int)'x')))
     {					/* tenth character may be [0-9xX] */
 	n_significant++;
 	return (YES);
@@ -776,6 +864,20 @@ size_t n;
 }
 
 
+#if defined(HAVE_STDC)
+static YESorNO
+is_ISBN_13_char(int c, size_t n)
+#else /* K&R style */
+static YESorNO
+is_ISBN_13_char(c,n)
+int c;
+size_t n;
+#endif
+{
+    return (is_ISBN_char(c, n));
+}
+
+
 #if defined(HAVE_STDC)
 static YESorNO
 is_ISSN_char(int c, size_t n)
@@ -791,21 +893,21 @@ size_t n;
 
     /* ISSNs match
 	[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9][- ]*
-        [0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9xX]
+	[0-9][- ]*[0-9][- ]*[0-9][- ]*[0-9xX]
     */
 
     if (n == 0)				/* start new ISSN */
 	n_significant = 0;
 
     /* embedded hyphens and space are accepted, but are not significant */
-    if ((n_significant > 0) && ((c == '-') || isspace(c)))
+    if ((n_significant > 0) && ((c == (int)'-') || Isspace(c)))
 	return (YES);
-    else if ((n_significant < 7) && isdigit(c))
+    else if ((n_significant < 7) && Isdigit(c))
     {
 	n_significant++;
 	return (YES);
     }
-    else if ((n_significant >= 7) && (isdigit(c) || (c == 'X') || (c == 'x')))
+    else if ((n_significant >= 7) && (Isdigit(c) || (c == (int)'X') || (c == (int)'x')))
     {					/* eighth character may be [0-9xX] */
 	n_significant++;
 	return (YES);
@@ -815,6 +917,20 @@ size_t n;
 }
 
 
+#if defined(HAVE_STDC)
+static YESorNO
+is_URL_char(int c, size_t n)
+#else /* K&R style */
+static YESorNO
+is_URL_char(c,n)
+int c;
+size_t n;
+#endif
+{
+    return (Isprint(c) ? YES : NO);    /* URLs match any printable string */
+}
+
+
 #if defined(HAVE_STDC)
 static void
 parse_list(const char *s, YESorNO (*is_name_char) ARGS((int c, size_t n)),
@@ -885,30 +1001,30 @@ void (*validate) ARGS((const char *s, size_t n));
 
 #if defined(HAVE_STDC)
 static void
-parse_element(parse_data *pd)
+parse_element(/*@out@*/ parse_data *pd)
 #else /* K&R style */
 static void
 parse_element(pd)
-parse_data *pd;
+/*@out@*/ parse_data *pd;
 #endif
 {
     size_t n;
 
-    for (n = 0, pd->token = pd->s; *pd->s && ((*pd->is_name_char)(*pd->s,n) == YES);
+    for (n = 0, pd->token = pd->s; (*pd->s != '\0') && ((*pd->is_name_char)((int)*pd->s,n) == YES);
 	 n++, pd->s++)
-	/* NO-OP */;
+	continue;
 
-    pd->token_length = (int)(pd->s - pd->token);
+    pd->token_length = (size_t)(pd->s - pd->token);
 }
 
 
 #if defined(HAVE_STDC)
 static void
-parse_separator(parse_data *pd)
+parse_separator(/*@out@*/ parse_data *pd)
 #else /* K&R style */
 static void
 parse_separator(pd)
-parse_data *pd;
+/*@out@*/ parse_data *pd;
 #endif
 {
     size_t n;
@@ -917,7 +1033,7 @@ parse_data *pd;
     pd->token = pd->s;
 
     for (n = 0, paren_level = 0;
-	 (*pd->s && (((*pd->is_name_char)(*pd->s,n) == NO) || (paren_level > 0)));
+	 ((*pd->s != '\0') && (((*pd->is_name_char)((int)*pd->s,n) == NO) || (paren_level > 0)));
 	n++, pd->s++)
     {
 	if (*pd->s == '(')
@@ -930,7 +1046,7 @@ parse_data *pd;
 	}
     }
 
-    pd->token_length = (int)(pd->s - pd->token);
+    pd->token_length = (size_t)(pd->s - pd->token);
     if (paren_level != 0)
 	warning("Non-zero parenthesis level in ``%f = %v''");
 }
@@ -943,8 +1059,15 @@ unexpected(VOID)
 }
 
 
+#if defined(HAVE_STDC)
 static void
 validate_CODEN(const char *the_CODEN, size_t n)
+#else
+static void
+validate_CODEN(the_CODEN, n)
+const char *the_CODEN;
+size_t n;
+#endif
 {
     int checksum;
     char CODEN[1 + MAX_CODEN + 1];	/* saved CODEN for error messages */
@@ -992,7 +1115,7 @@ validate_CODEN(const char *the_CODEN, size_t n)
     (void)strcpy(&CODEN[1], UNKNOWN_CODEN);
     nleft = copy_element(&CODEN[1], sizeof(CODEN)-1, the_CODEN, n);
 
-    for (checksum = 0, k = 1; CODEN[k]; ++k)
+    for (checksum = 0, k = 1; CODEN[k] != '\0'; ++k)
     {
 	if (k < MAX_CODEN)
 	{
@@ -1017,8 +1140,58 @@ validate_CODEN(const char *the_CODEN, size_t n)
 }
 
 
+#if defined(HAVE_STDC)
+static void
+validate_DOI(const char *the_DOI, size_t n)
+#else
+static void
+validate_DOI(the_DOI, n)
+const char *the_DOI;
+size_t n;
+#endif
+{
+    /*
+    ** Typical input:
+    **
+    **     the_DOI = "\"http://dx.doi.org/10.1000/a.b.c\""
+    **
+    ** Notice that the delimiting quotation marks are part of the
+    ** string value, so we must skip the first character in matching
+    ** against members of the list of valid prefixes.
+    */
+
+    int k, match_count;
+    static const char *doi_prefix_list[] =
+    {	/* the DOI organization introduced new prefixes in 2016 */
+	"http://doi.org/",
+	"http://dx.doi.org/",	/* original single-prefix style (2000--2016) */
+	"https://doi.org/",
+	"https://dx.doi.org/",
+#if defined(DOI_RAW_VALID)
+	"10."
+#endif
+    };
+
+    for (k = 0, match_count = 0; k < elementsof(doi_prefix_list); ++k)
+    {
+	if (strncmp(&the_DOI[1], doi_prefix_list[k], strlen(doi_prefix_list[k])) == 0)
+	    match_count++;
+    }
+
+    if (match_count == 0)
+	warning("Unexpected prefix in DOI value ``%v''");
+}
+
+
+#if defined(HAVE_STDC)
 static void
 validate_ISBN(const char *the_ISBN, size_t n)
+#else
+static void
+validate_ISBN(the_ISBN, n)
+const char *the_ISBN;
+size_t n;
+#endif
 {
     int checksum;
     char ISBN[1 + MAX_ISBN + 1];	/* saved ISBN for error messages */
@@ -1044,7 +1217,7 @@ validate_ISBN(const char *the_ISBN, size_t n)
     (void)strcpy(&ISBN[1],UNKNOWN_ISBN);
     nleft = copy_element(&ISBN[1], sizeof(ISBN)-1, the_ISBN, n);
 
-    for (checksum = 0, k = 1; ISBN[k]; ++k)
+    for (checksum = 0, k = 1; ISBN[k] != '\0'; ++k)
     {
 	if (k < MAX_ISBN)
 	    checksum += ISBN_DIGIT_VALUE(ISBN[k]) * k;
@@ -1060,10 +1233,83 @@ validate_ISBN(const char *the_ISBN, size_t n)
 }
 
 
+#if defined(HAVE_STDC)
 static void
-validate_ISSN(const char *the_ISSN, size_t n)
+validate_ISBN_13(const char *the_ISBN_13, size_t n)
+#else
+static void
+validate_ISBN_13(the_ISBN_13, n)
+const char *the_ISBN_13;
+size_t n;
+#endif
 {
     int checksum;
+    char ISBN_13[1 + MAX_ISBN_13 + 1];	/* saved ISBN_13 for error messages */
+					/* (use slots 1..13 instead of 0..12) */
+    size_t k;				/* index into ISBN_13[] */
+    size_t nleft;
+
+    /*******************************************************************
+       ISBN_13 numbers are 13-character values from the set [0-9Xx], with
+       a final checksum digit given by
+
+		rem = (sum(k=1:12) digit(k) * weight(k)) mod 10
+		weight(k) = if (k odd) then 1 else 3
+		digit(13) = if (rem == 0) then 0 else (10 - rem)
+
+       where digits have their normal value, X (or x) as a digit has
+       value 10, and spaces and hyphens are ignored.  The sum is
+       bounded from above by 3*(9 + 9 + ... + 9) = 324, so even
+       short (16-bit) integers are sufficient for the accumulation.
+
+       ISBN_13 digits are grouped into five parts separated by space
+       or hyphen:
+
+	   978-countrygroupnumber-publishernumber-booknumber-checkdigit.
+
+       The initial prefix changes to 979 when the 978 group is
+       exhausted.
+    *******************************************************************/
+
+    (void)strcpy(&ISBN_13[1],UNKNOWN_ISBN_13);
+    nleft = copy_element(&ISBN_13[1], sizeof(ISBN_13)-1, the_ISBN_13, n);
+
+    for (checksum = 0, k = 1; ISBN_13[k] != '\0'; ++k)
+    {
+	size_t weight;
+
+	weight = (k & 1) ? 1 : 3;
+
+	if (k < MAX_ISBN_13)
+	    checksum += ISBN_DIGIT_VALUE(ISBN_13[k]) * weight ;
+	else if (k == MAX_ISBN_13)
+	{
+	    size_t digit_13, rem;
+
+	    rem = checksum % 10;
+	    digit_13 = (rem == 0) ? 0 : (10 - rem);
+
+	    if (digit_13 != ISBN_DIGIT_VALUE(ISBN_13[k]))
+		bad_ISBN_13(ISBN_13);
+	}
+    } 					/* end for (loop over ISBN_13[]) */
+
+    if ((strlen(&ISBN_13[1]) != MAX_ISBN_13) || (nleft > 0))
+	bad_ISBN_13(ISBN_13);
+}
+
+
+#if defined(HAVE_STDC)
+static void
+validate_ISSN(const char *the_ISSN, size_t n)
+#else
+static void
+validate_ISSN(the_ISSN, n)
+const char *the_ISSN;
+size_t n;
+#endif
+{
+    long checksum;
     char ISSN[1 + MAX_ISSN + 1];	/* saved ISSN for error messages */
 					/* (use slots 1..8 instead of 0..7) */
     size_t k;				/* index into ISSN[] */
@@ -1087,13 +1333,13 @@ validate_ISSN(const char *the_ISSN, size_t n)
     (void)strcpy(&ISSN[1],UNKNOWN_ISSN);
     nleft = copy_element(&ISSN[1], sizeof(ISSN)-1, the_ISSN, n);
 
-    for (checksum = 0, k = 1; ISSN[k]; ++k)
+    for (checksum = 0L, k = 1; (ISSN[k] != '\0'); ++k)
     {
 	if (k < MAX_ISSN)
-	    checksum += ISSN_DIGIT_VALUE(ISSN[k]) * (k + 2);
+	    checksum += (long)(ISSN_DIGIT_VALUE(ISSN[k]) * (k + 2));
 	else if (k == MAX_ISSN)
 	{
-	    if ((checksum % 11) != ISSN_DIGIT_VALUE(ISSN[k]))
+	    if ((checksum % 11L) != ISSN_DIGIT_VALUE(ISSN[k]))
 		bad_ISSN(ISSN);
 	}
     } 					/* end for (loop over ISSN[]) */
@@ -1101,3 +1347,42 @@ validate_ISSN(const char *the_ISSN, size_t n)
     if ((strlen(&ISSN[1]) != MAX_ISSN) || (nleft > 0))
 	bad_ISSN(ISSN);
 }
+
+#if defined(HAVE_STDC)
+static void
+validate_URL(const char *the_URL, size_t n)
+#else
+static void
+validate_URL(the_URL, n)
+const char *the_URL;
+size_t n;
+#endif
+{
+    char *p;
+
+    p = stristr(the_URL, "://");
+
+    if (p == (char *)NULL)
+	warning("Expected protocol://... in URL value ``%v%''");
+    else
+    {
+	if ( ((p - the_URL) >= 3) && (strncmp(&p[-3], "ftp", 3) == 0) )
+	    /* NO-OP */ ;
+	else if ( ((p - the_URL) >= 4) && (strncmp(&p[-4], "http", 4) == 0) )
+	    /* NO-OP */ ;
+	else if ( ((p - the_URL) >= 5) && (strncmp(&p[-5], "https", 5) == 0) )
+	    /* NO-OP */ ;
+	else
+	    warning("Unexpected protocol://... in URL value ``%v'': normally ftp://... or http://... or https://...");
+
+	if (stristr(the_URL, "doi.org/") != (char *)NULL)
+	    warning("Unexpected DOI in URL value ``%v'': move to separate DOI = \"...\" key/value in this entry");
+	else if ( (stristr(the_URL, ".com/10.") != (char *)NULL) ||
+		  (stristr(the_URL, ".edu/10.") != (char *)NULL) ||
+		  (stristr(the_URL, ".gov/10.") != (char *)NULL) ||
+		  (stristr(the_URL, ".mil/10.") != (char *)NULL) ||
+		  (stristr(the_URL, ".net/10.") != (char *)NULL) ||
+		  (stristr(the_URL, ".org/10.") != (char *)NULL) )
+	    warning("Possible DOI in URL value ``%v'': if so, move to separate DOI = \"...\" key/value in this entry");
+    }
+}
