@@ -6,7 +6,7 @@
 
 /***********************************************************************
 NB:  If  the macro DVI  is defined  when this  file  is compiled, then
-additional code will be executed  in isfile() to  optionally trace the
+additional code will be executed  in is_file() to  optionally trace the
 lookup attempts.
 
 If the macro TEST is defined at compile-time, a test main program will
@@ -124,14 +124,21 @@ substituted name, so it can be later used to open the file.
 #include "xunistd.h"
 #include "xctype.h"
 #include "xpwd.h"			/* for ~name lookup in envsub() */
+#include "xstdbool.h"
 #include "xstdlib.h"
 #include "xstring.h"
 
+#include "ch.h"
+#include "yesorno.h"
+
 #if defined(HAVE_IO_H)
 #include <io.h>				/* for more function declarations */
 #endif
 
-RCSID("$Id: fndfil.c,v 1.2 1996/05/03 20:15:28 beebe Exp beebe $")
+RCSID("$Id: fndfil.c,v 1.7 2014/04/03 18:04:55 beebe Exp beebe $")
+
+extern FILE		*stdlog;
+extern YESorNO		trace_file_opening;
 
 #ifdef DVI
 #include "typedefs.h"
@@ -148,10 +155,10 @@ extern char		g_logname[];	 /* name of log file, if created */
 #if !defined(R_OK)
 #define R_OK 4
 #endif
-#define FILE_IS_READABLE(n) ((int)access((char*)n,R_OK) == 0)
+#define IS_FILE_READABLE(n) is_file_readable(n)
 
-#define ISNAMEPREFIX(c) (isalpha(c) || ((c) == '_'))
-#define ISNAMESUFFIX(c) (isalnum(c) || ((c) == '_'))
+#define ISNAMEPREFIX(c) (Isalpha((int)(c)) || ((int)(c) == (int)'_'))
+#define ISNAMESUFFIX(c) (Isalnum((int)(c)) || ((int)(c) == (int)'_'))
 
 #ifdef MIN
 #undef MIN
@@ -171,13 +178,14 @@ declares it correctly. */
 #define STRSPN(s,t)	(size_t)strspn(s,t)
 
 extern void		dbglookup ARGS((const char*, const char*));
-extern char		*findfile ARGS((const char *pathlist_,
-					const char *name_));
+/*@null@*/ extern char	*findfile ARGS((/*@null@*/ const char *pathlist_,
+					/*@null@*/ const char *name_));
 extern void		warning ARGS((const char *msg_));
 
 static const char	*copyname ARGS((char *target_, const char *source_));
-static char		*envsub ARGS((const char *filename_));
-static int		isfile ARGS((char *filename_));
+static bool		is_file_readable ARGS((const char *filename));
+/*@null@*/ static char	*envsub ARGS((const char *filename_));
+static bool		is_file ARGS((char *filename_));
 
 
 #ifdef TEST
@@ -185,7 +193,7 @@ int		main ARGS((void));
 #endif /* TEST */
 
 /***********************************************************************
-Copy environment variable or usename, and return updated pointer past
+Copy environment variable or username, and return updated pointer past
 end of copied name in source[].
 ***********************************************************************/
 
@@ -217,7 +225,7 @@ return  (char*)NULL if no  changes were made,  and otherwise, return a
 pointer to the internal copy of the expanded filename.
 ***********************************************************************/
 
-static char*
+/*@null@*/ static char*
 #if defined(HAVE_STDC)
 envsub(
 const char			*filename
@@ -235,7 +243,7 @@ const char			*filename;
     register const char		*s;
     char			tmpfname[MAXPATHLEN+1];
 
-#if    OS_UNIX
+#if    OS_UNIX && defined(HAVE_GETPWNAM)
     struct passwd		*pw;	/* handle tilde processing */
 
     tmpfname[0] = '\0';			/* initialize tmpfname[] */
@@ -243,14 +251,17 @@ const char			*filename;
     if (p != (char*)NULL)		/* handle tilde (once per filename) */
     {
 	(void)strncpy(tmpfname,filename,(size_t)(p - filename));
-	tmpfname[(int)(p - filename)] = '\0'; /* terminate copied string */
+	tmpfname[(size_t)(p - filename)] = '\0'; /* terminate copied string */
 	r = strchr(tmpfname,'\0');	/* remember start of name */
 	++p;				/* point past tilde */
 	if (ISNAMEPREFIX(*p))		/* expect ~name */
 	{
 	    p = copyname(r,p);		/* p now points past ~name */
 	    pw = getpwnam(r);		/* get password structure */
-	    (void)strcpy(r,pw->pw_dir);	/* replace name by login directory */
+	    if (pw == (struct passwd *)NULL)
+		p--;			/* restore p to point to tilde */
+	    else
+		(void)strcpy(r,pw->pw_dir);	/* replace name by login directory */
 	}
 	else				/* expect ~/name */
 	    (void)strcat(tmpfname,"${HOME}"); /* change to ${HOME}/name */
@@ -262,25 +273,31 @@ const char			*filename;
     (void)strcpy(tmpfname,filename);	/* copy of filename */
 #endif /* OS_UNIX */
 
-    for (r = altname, *r = '\0', p = tmpfname; *p; )
+    for (r = altname, *r = '\0', p = tmpfname; *p != '\0'; )
     {					/* handle environment variables */
 	if (*p == '$')			/* expect $NAME or ${NAME} */
 	{
 	    p++;			/* point past $ */
+
 	    if (*p == '{')		/* have ${NAME} */
 	    {
 		p = copyname(r,p+1);
+
 		if (*p++ != '}')
 		    return ((char*)NULL); /* bail out -- no closing brace */
-		for (s = (const char *)getenv(r); (s != (char*)NULL) && *s;)
+
+		for (s = (const char *)getenv(r); (s != (char*)NULL) && (*s != '\0');)
 		    *r++ = *s++;	/* copy environment variable value */
+
 		*r = '\0';		/* terminate altname[] */
 	    }
 	    else if (ISNAMEPREFIX(*p))	/* have $NAME */
 	    {
 		p = copyname(r,p);
-		for (s = (const char *)getenv(r); (s != (char*)NULL) && *s;)
+
+		for (s = (const char *)getenv(r); (s != (char*)NULL) && (*s != '\0');)
 		    *r++ = *s++;	/* copy environment variable value */
+
 		*r = '\0';		/* terminate altname[] */
 	    }
 	    else			/* invalid $NAME */
@@ -289,6 +306,7 @@ const char			*filename;
 	else				/* just copy character */
 	    *r++ = *p++;
     }
+
     *r = '\0';				/* terminate altname[] */
 
 #ifdef TEST
@@ -296,20 +314,25 @@ const char			*filename;
     for (p = filename; *p; ++p)
     {
 	putchar(*p);
-	if (strchr(SEP_COMP,*p) != (char*)NULL)
+
+	if (IN_SET(SEP_COMP,*p))
 	    (void)printf("\n\t");
     }
+
     (void)printf("] ->\n\t[");
+
     for (p = altname; *p; ++p)
     {
 	putchar(*p);
-	if (strchr(SEP_COMP,*p) != (char*)NULL)
+
+	if (IN_SET(SEP_COMP,*p))
 	    (void)printf("\n\t");
     }
+
     (void)printf("]\n");
 #endif /* TEST */
 
-    return (altname[0] ? altname : (char*)NULL);
+    return ((altname[0] != '\0') ? altname : (char*)NULL);
 #else  /* NOT (OS_ATARI || OS_PCDOS || OS_UNIX) */
     return ((char*)NULL);		/* no processing to be done */
 #endif /* (OS_ATARI || OS_PCDOS || OS_UNIX) */
@@ -317,6 +340,31 @@ const char			*filename;
 }
 
 
+static bool
+#if defined(HAVE_STDC)
+is_file_readable(
+const char  *filename
+)
+#else /* NOT defined(HAVE_STDC) */
+is_file_readable(filename)
+const char  *filename;
+#endif /* defined(HAVE_STDC) */
+{
+    /* return true if filename is readable, and false otherwise */
+    bool is_readable;
+
+    is_readable = ((int)access((char*)filename,R_OK) == 0) ? true : false;
+
+    if (trace_file_opening == YES)
+    {
+	(void)fprintf(stdlog,"%s lookup file [%s]%s\n",
+	    WARNING_PREFIX, filename, is_readable ? "" : ": FAILED");
+    }
+
+    return (is_readable);
+}
+
+
 /***********************************************************************
 Given a directory search path in pathlist[] and a file name in name[],
 search the path  for an  existing file.  If  one  is  found, return  a
@@ -324,23 +372,23 @@ pointer  to  an  internal copy   of its  full  name; otherwise, return
 (char*)NULL.
 ***********************************************************************/
 
-char*
+/*@null@*/ char*
 #if defined(HAVE_STDC)
 findfile(
-const char  *pathlist,
-const char  *name
+/*@null@*/ const char  *pathlist,
+/*@null@*/ const char  *name
 )
 #else /* NOT defined(HAVE_STDC) */
 findfile(pathlist,name)
-const char  *pathlist;
-const char  *name;
+/*@null@*/ const char  *pathlist;
+/*@null@*/ const char  *name;
 #endif /* defined(HAVE_STDC) */
 {
 #define BADCACHEFMT	"Invalid font cache entry: font %s is not accessible"
     static char			fullname[MAXPATHLEN+1];
     static char			fullpath[MAXPATHLEN+1];
-    int				k;
-    int				len;
+    size_t			k;
+    size_t			len;
     register const char		*p;
 
 #ifdef DVI
@@ -353,16 +401,20 @@ const char  *name;
     if (fsf_table != (HASH_TABLE*)NULL)
     {
 	he = hash_lookup(name, fsf_table); /* name found in filename cache? */
+
 	if ( (he != (HASH_ENTRY*)NULL) && (he->hash_key != (const char*)NULL) )
 	{				/* yes, reconstruct the full filename */
 	    (void)strcpy(fullname,(const char*)he->hash_data);
-	    k = (int)strlen(fullname);
-	    if (strchr(SEP_PATH,fullname[k-1]) == (char*)NULL)
+	    k = strlen(fullname);
+
+	    if ( (k > 0) && !IN_SET(SEP_PATH, fullname[k-1]) )
 		fullname[k++] = SEP_PATH[0];	/* supply directory separator */
+
 	    (void)strncpy(&fullname[k],name,(size_t)(MAXPATHLEN-k));
 					/* append name */
 	    fullname[MAXPATHLEN] = '\0'; /* strncpy may not supply this */
 	    dbglookup(fullname, "CACHED");
+
 	    /***********************************************************
 	      Originally, we returned the filename without an access
 	      check, but occasionally, this caused problems if the cache
@@ -370,8 +422,10 @@ const char  *name;
 	      warning if the file is inaccessible, either because it
 	      does not exist at all, or lacks read permission.
 	    ***********************************************************/
-	    if (FILE_IS_READABLE(fullname))
+
+	    if (IS_FILE_READABLE(fullname))
 		return ((char*)&fullname[0]);
+
 	    (void)sprintf(message,BADCACHEFMT,fullname);
 	    warning(message);
 	}
@@ -407,7 +461,7 @@ const char  *name;
 	for (k = 0; k < 10; ++k)
 	{
 	    p = envsub(fullpath);
-	    if ( (p == (char*)NULL) || (strcmp(p,fullpath) == 0) )
+	    if ( (p == (char*)NULL) || STREQUAL(p,fullpath) )
 		break;			/* no new substitutions were made */
 	    else			/* save new expansion */
 		(void)strcpy(fullpath,p);
@@ -419,20 +473,23 @@ const char  *name;
     {
 	(void)strncpy(fullname,name,MAXPATHLEN);
 	fullname[MAXPATHLEN] = '\0';	/* in case strncpy() doesn't do it */
-	return (isfile(fullname) ? (char*)&fullname[0] : (char*)NULL);
+	return (is_file(fullname) ? (char*)&fullname[0] : (char*)NULL);
     }
-    while (*p)				/* process pathlist */
+    while (*p != '\0')				/* process pathlist */
     {
-	len = (int)strcspn(p,SEP_COMP);	/* get length before separator */
+	len = strcspn(p,SEP_COMP);	/* get length before separator */
 	len = MIN(MAXPATHLEN-1,len);	/* leave space for possible SEP_PATH */
-	(void)strncpy(fullname,p,(size_t)len); /* set directory name */
+	(void)strncpy(fullname,p,len);	/* set directory name */
 	k = len;
-	if (strchr(SEP_PATH,fullname[k-1]) == (char*)NULL)
+
+	if ( (k > 0) && !IN_SET(SEP_PATH, fullname[k - 1]) )
 	    fullname[k++] = SEP_PATH[0];	/* supply directory separator */
+
 	(void)strncpy(&fullname[k],name,(size_t)(MAXPATHLEN-k));
 					/* append name */
 	fullname[MAXPATHLEN] = '\0';	/* strncpy may not supply this */
-	if (isfile(fullname))
+
+	if (is_file(fullname))
 	    return ((char*)&fullname[0]);
 
 #if    OS_VAXVMS
@@ -441,7 +498,7 @@ const char  *name;
 	    char		*logname;
 	    int			n;
 
-	    if (fullname[k-1] == ']')	/* then not logical name */
+	    if ((k > 0) && (fullname[k-1] == ']')) /* then not logical name */
 		break;			/* here's a loop exit */
 	    fullname[k] = '\0';		/* terminate logical name */
 	    logname = GETENV(fullname);
@@ -475,14 +532,14 @@ const char  *name;
 		    fullname[n++] = ':';
 		(void)strcpy(&fullname[n],&name[0]); /* logname:name */
 	    }
-	    if (isfile(fullname))
+	    if (is_file(fullname))
 		return ((char*)&fullname[0]);
 	}
 	while (0);
 #endif /* OS_VAXVMS */
 
 	p += len;			/* point past first path */
-	if (*p)				/* then not at end of path list */
+	if (*p != '\0')			/* then not at end of path list */
 	{
 	    p++;			/* point past separator */
 	    p += STRSPN(p,SEP_COMP);	/* skip over any extra separators */
@@ -501,27 +558,27 @@ we  replace filename with the  substituted  name,  since  that will be
 needed later to open the file.
 ***********************************************************************/
 
-static int
+static bool
 #if defined(HAVE_STDC)
-isfile(
+is_file(
 char				*filename
 )
 #else /* NOT defined(HAVE_STDC) */
-isfile(filename)
+is_file(filename)
 char				*filename;
 #endif /* defined(HAVE_STDC) */
 {
-    int				exists;
+    bool				exists;
     char			*p;
 
-    exists = FILE_IS_READABLE(filename);
+    exists = IS_FILE_READABLE(filename);
 
     if (!exists)		/* try environment variable substitution */
     {
 	p = envsub(filename);
 	if (p != (char*)NULL)
 	{
-	    exists = FILE_IS_READABLE(p);
+	    exists = IS_FILE_READABLE(p);
 	    if (exists)
 		(void)strcpy(filename,p);
 	}
