@@ -1,29 +1,32 @@
 #include <config.h>
+#include "xlimits.h"
+#include "xstdbool.h"
 #include "xstdlib.h"
 #include "xstring.h"
 
-RCSID("$Id: option.c,v 1.7 1999/05/20 13:26:59 beebe Exp beebe $")
+RCSID("$Id: option.c,v 1.14 2014/04/03 18:07:03 beebe Exp beebe $")
 
 #include "ch.h"
+#include "isbn.h"
 #include "yesorno.h"
-
-#define BIBCLEAN_VERSION	"bibclean Version 2.11.4 [09-May-1998]"
-
-typedef struct s_option_function_entry
-{
-    const char *name;		/* option name */
-    size_t min_match;		/* minimum length string match */
-    void (*function)(VOID);	/* function to call when option matched */
-} OPTION_FUNCTION_ENTRY;
+#include "match.h"
+#include "typedefs.h"			/* must come AFTER match.h */
 
 extern YESorNO	align_equals;		/* NO: left-adjust equals */
+extern YESorNO	brace_protect;		/* YES: brace-protect title words */
 extern YESorNO	check_values;		/* NO: suppress value checks */
+extern YESorNO	debug_match_failures; 	/* YES: debug value pattern match failures */
 extern YESorNO	delete_empty_values; 	/* YES: delete empty values */
-extern YESorNO	fix_initials;		/* reformat A.U. Thor?	*/
-extern YESorNO	fix_names;		/* reformat Bach, P.D.Q? */
+extern YESorNO	fix_accents;		/* repair accent bracing? */
+extern YESorNO	fix_braces;		/* normalize bracing? */
+extern YESorNO	fix_degrees;		/* normalize author academic degrees? */
 extern YESorNO	fix_font_changes;	/* brace {\em E. Coli}? */
+extern YESorNO	fix_initials;		/* reformat A.U. Thor?	*/
+extern YESorNO	fix_math;		/* reformat math mode? */
+extern YESorNO	fix_names;		/* reformat Bach, P.D.Q? */
+
 extern YESorNO	German_style;		/* YES: " inside braced string
-						value obeys german.sty style */
+					   value obeys german.sty style */
 extern YESorNO	keep_linebreaks;	/* YES: keep linebreaks in values */
 extern YESorNO	keep_parbreaks;		/* YES: keep parbreaks in values */
 extern YESorNO	keep_preamble_spaces;	/* YES: keep spaces in @Preamble{} */
@@ -33,6 +36,8 @@ extern long	max_width;
 extern YESorNO	parbreaks;		/* NO: parbreaks forbidden */
 					/* in strings and entries */
 extern YESorNO	prettyprint;		/* NO: do lexical analysis */
+extern YESorNO	print_ISBN_table;	/* YES: print ISBN table */
+extern YESorNO	print_keyword_table;	/* YES: print keyword table */
 extern YESorNO	print_patterns;		/* YES: print value patterns */
 extern char	*program_name;		/* set to argv[0] */
 extern YESorNO	read_initialization_files; /* -[no-]read-init-files sets */
@@ -41,26 +46,25 @@ extern int	screen_lines;		/* kbopen() and out_lines() reset */
 extern YESorNO	Scribe;			/* Scribe format input */
 extern YESorNO	show_file_position; 	/* messages usually brief */
 extern FILE	*stdlog;		/* usually stderr */
-extern YESorNO	trace_file_opening; /* -[no-]trace-file-opening sets */
+extern YESorNO	trace_file_opening;	/* -[no-]trace-file-opening sets */
 extern YESorNO	warnings;		/* NO: suppress warnings */
 
-static int	current_index;			/* argv[] index in do_args() */
-static char	*current_option;		/* set by do_args() */
-static char	*next_option;			/* set in do_args() */
+static int	current_index;		/* argv[] index in do_args() */
+static char	*current_option;	/* set by do_args() */
+static char	*next_option;		/* set in do_args() */
 
 void		do_args ARGS((int argc_, char *argv_[]));
 void		do_preargs ARGS((int argc_, char *argv_[]));
-int		isoptionprefix ARGS((int c));
+
+bool		is_optionprefix ARGS((int c));
 
 extern YESorNO	apply_function ARGS((const char *option_,
 		    OPTION_FUNCTION_ENTRY table_[]));
 extern void	check_inodes ARGS((void));
 extern void	do_initfile ARGS((const char *pathlist_,const char *name_));
-
-#if (SCREEN_LINES > 0)
-extern int	get_screen_lines ARGS((void));
-#endif /* (SCREEN_LINES > 0) */
-
+extern void	do_keyword_file ARGS((const char *pathlist_,const char *name_));
+extern void	finish ARGS((int status));
+extern void	free_keyword_table ARGS((void));
 extern void	out_lines ARGS((FILE *fpout_,const char *lines_[],
 		    YESorNO pausing_));
 extern FILE	*tfopen ARGS((const char *filename_, const char *mode_));
@@ -68,25 +72,38 @@ extern void	warning ARGS((const char *msg_));
 
 static void	opt_align_equals ARGS((void));
 static void	opt_author ARGS((void));
+static void	opt_brace_protect ARGS((void));
 static void	opt_check_values ARGS((void));
+static void	opt_copyleft ARGS((void));
+static void	opt_debug_match_failures ARGS((void));
 static void	opt_delete_empty_values ARGS((void));
 static void	opt_error_log ARGS((void));
 static void	opt_file_position ARGS((void));
+static void	opt_fix_accents ARGS((void));
+static void	opt_fix_braces ARGS((void));
+static void	opt_fix_degrees ARGS((void));
 static void	opt_fix_font_changes ARGS((void));
 static void	opt_fix_initials ARGS((void));
+static void	opt_fix_math ARGS((void));
 static void	opt_fix_names ARGS((void));
 static void	opt_help ARGS((void));
 static void	opt_init_file ARGS((void));
+static void	opt_ISBN_file ARGS((void));
 static void	opt_German_style ARGS((void));
 static void	opt_keep_linebreaks ARGS((void));
 static void	opt_keep_parbreaks ARGS((void));
 static void	opt_keep_preamble_spaces ARGS((void));
 static void	opt_keep_spaces ARGS((void));
 static void	opt_keep_string_spaces ARGS((void));
+static void	opt_keyword_file ARGS((void));
 static void	opt_max_width ARGS((void));
+static void	opt_output_file ARGS((void));
 static void	opt_parbreaks ARGS((void));
 static void	opt_prettyprint ARGS((void));
+static void	opt_print_ISBN_table ARGS((void));
+static void	opt_print_keyword_table ARGS((void));
 static void	opt_print_patterns ARGS((void));
+static void	opt_quiet ARGS((void));
 static void	opt_read_init_files ARGS((void));
 static void	opt_remove_OPT_prefixes ARGS((void));
 static void	opt_scribe ARGS((void));
@@ -97,6 +114,35 @@ static void	usage ARGS((void));
 static void	version ARGS((void));
 static YESorNO	YESorNOarg ARGS((void));
 
+static CONST char *help_lines[] =
+{
+    "\nUsage: ",
+    (const char*)NULL,
+    " [ -author ] [ -copyleft ] [ -copyright ]\n",
+    "\t[ -error-log filename ] [ -help ] [ '-?'  ]\n",
+    "\t[ -init-file filename ] [ -ISBN-file filename ]\n",
+    "\t[ -keyword-file filename ] [ -max-width nnn ]\n",
+    "\t[ -[no-]align-equals ] [ -[no-]brace-protect ]\n",
+    "\t[ -[no-]check-values ] [ -[no-]debug-match-failures ]\n",
+    "\t[ -[no-]delete-empty-values ] [ -[no-]file-position ]\n",
+    "\t[ -[no-]fix-accents ] [ -[no-]fix-braces ] [ -[no-]fix-degrees ]\n",
+    "\t[ -[no-]fix-font-changes ] [ -[no-]fix-initials ]\n",
+    "\t[ -[no-]fix-math ] [ -[no-]fix-names ] [ -[no-]German-style ]\n",
+    "\t[ -[no-]keep-linebreaks ] [ -[no-]keep-parbreaks ]\n",
+    "\t[ -[no-]keep-preamble-spaces ] [ -[no-]keep-spaces ]\n",
+    "\t[ -[no-]keep-string-spaces ] [ -[no-]parbreaks ]\n",
+    "\t[ -[no-]prettyprint ] [ -[no-]print-ISBN-table ]\n",
+    "\t[ -[no-]print-keyword-table ] [ -[no-]print-patterns ]\n",
+    "\t[ -[no-]quiet ] [ -[no-]read-init-files ]\n",
+    "\t[ -[no-]remove-OPT-prefixes ] [ -[no-]scribe ]\n",
+    "\t[ -[no-]trace-file-opening ] [ -[no-]warnings ]\n",
+    "\t[ -output-file filename ] [ -version ]\n",
+    "\t[ <infile or bibfile1 bibfile2 bibfile3 ...] >outfile\n",
+    "\n",
+#include "bibclean.h"
+};
+
+
 #if defined(HAVE_STDC)
 void
 do_args(int argc, char *argv[])
@@ -119,44 +165,69 @@ char *argv[];
 	{"?",				1,	opt_help},
 	{"align-equals",		2,	opt_align_equals},
 	{"author",			2,	opt_author},
+	{"brace-protect",		1,	opt_brace_protect },
 	{"check-values",		1,	opt_check_values},
+	{"copyleft",			2,	opt_copyleft },
+	{"copyright",			2,	opt_copyleft },
+	{"debug-match-failures",	3,	opt_debug_match_failures},
 	{"delete-empty-values",		1,	opt_delete_empty_values},
 	{"error-log",			1,	opt_error_log},
 	{"file-position",		3,	opt_file_position},
+	{"fix-accents",			5,	opt_fix_accents},
+	{"fix-braces",			5,	opt_fix_braces},
+	{"fix-degrees",			5,	opt_fix_degrees},
 	{"fix-font-changes",		5,	opt_fix_font_changes},
 	{"fix-initials",		5,	opt_fix_initials},
+	{"fix-math",			5,	opt_fix_math},
 	{"fix-names",			5,	opt_fix_names},
 	{"German-style",		1,	opt_German_style},
 	{"help",			1,	opt_help},
-	{"init-file",			1,	opt_init_file},
+	{"init-file",			2,	opt_init_file},
+	{"ISBN-file",			2,	opt_ISBN_file},
 	{"keep-linebreaks",		6,	opt_keep_linebreaks},
 	{"keep-parbreaks",		7,	opt_keep_parbreaks},
 	{"keep-preamble-spaces",	7,	opt_keep_preamble_spaces},
 	{"keep-spaces",			7,	opt_keep_spaces},
 	{"keep-string-spaces",		7,	opt_keep_string_spaces},
+	{"keyword-file",		3,	opt_keyword_file},
 	{"max-width",			1,	opt_max_width},
 	{"no-align-equals",		4,	opt_align_equals},
+	{"no-brace-protect",		4,	opt_brace_protect },
 	{"no-check-values",		4,	opt_check_values},
-	{"no-delete-empty-values",	4,	opt_delete_empty_values},
+	{"no-debug-match-failures",	6,	opt_debug_match_failures},
+	{"no-delete-empty-values",	6,	opt_delete_empty_values},
 	{"no-file-position",		6,	opt_file_position},
+	{"no-fix-accents",		8,	opt_fix_accents},
+	{"no-fix-braces",		8,	opt_fix_braces},
+	{"no-fix-degrees",		8,	opt_fix_degrees},
 	{"no-fix-font-changes",		8,	opt_fix_font_changes},
 	{"no-fix-initials",		8,	opt_fix_initials},
+	{"no-fix-math",			8,	opt_fix_math},
 	{"no-fix-names",		8,	opt_fix_names},
 	{"no-German-style",		4,	opt_German_style},
 	{"no-keep-linebreaks",		9,	opt_keep_linebreaks},
-	{"no-keep-parbreaks",		9,	opt_keep_parbreaks},
-	{"no-keep-spaces",		9,	opt_keep_spaces},
+	{"no-keep-parbreaks",		10,	opt_keep_parbreaks},
+	{"no-keep-preamble-spaces",	10,	opt_keep_preamble_spaces},
+	{"no-keep-spaces",		10,	opt_keep_spaces},
+	{"no-keep-string-spaces",	10,	opt_keep_string_spaces},
 	{"no-parbreaks",		5,	opt_parbreaks},
 	{"no-prettyprint",		6,	opt_prettyprint},
-	{"no-print-patterns",		6,	opt_print_patterns},
+	{"no-print-ISBN-table",		10,	opt_print_ISBN_table},
+	{"no-print-keyword-table",	10,	opt_print_keyword_table},
+	{"no-print-patterns",		10,	opt_print_patterns},
+	{"no-quiet",			4,	opt_quiet},
 	{"no-read-init-files",		6,	opt_read_init_files},
 	{"no-remove-OPT-prefixes",	6,	opt_remove_OPT_prefixes},
 	{"no-scribe",			4,	opt_scribe},
 	{"no-trace-file-opening",	4,	opt_trace_file_opening},
 	{"no-warnings",			4,	opt_warnings},
+	{"output-file",			1,	opt_output_file},
 	{"parbreaks",			2,	opt_parbreaks},
 	{"prettyprint",			3,	opt_prettyprint},
-	{"print-patterns",		3,	opt_print_patterns},
+	{"print-ISBN-table",		7,	opt_print_ISBN_table},
+	{"print-keyword-table",		7,	opt_print_keyword_table},
+	{"print-patterns",		7,	opt_print_patterns},
+	{"quiet",			1,	opt_quiet},
 	{"read-init-files",		3,	opt_read_init_files},
 	{"remove-OPT-prefixes",		3,	opt_remove_OPT_prefixes},
 	{"scribe",			1,	opt_scribe},
@@ -168,12 +239,12 @@ char *argv[];
 
     for (nfiles = 1, k = 1; k < argc; ++k)
     {
-	if ( (argv[k][1] != '\0') && isoptionprefix(argv[k][0]) )
+	if ( (argv[k][1] != '\0') && is_optionprefix((int)argv[k][0]) )
 	{				/* then process command-line switch */
 	    current_index = k;		/* needed by opt_init_file() and */
 	    next_option = argv[k+1];	/* opt_error_log() */
 	    current_option = argv[k];	/* needed by YESorNOarg() */
-	    if (isoptionprefix(current_option[1]))
+	    if (is_optionprefix((int)current_option[1]))
 		current_option++;	/* allow GNU/POSIX --option */
 	    if (apply_function(current_option+1,options) == NO)
 	    {
@@ -181,7 +252,7 @@ char *argv[];
 			      current_option);
 		warning(msg);
 		usage();
-		exit(EXIT_FAILURE);
+		finish(EXIT_FAILURE);
 	    }
 	    k = current_index;		/* some opt_xxx() functions update it */
 	}
@@ -206,10 +277,12 @@ char *argv[];
 
     static OPTION_FUNCTION_ENTRY options[] =
     {
-	{"no-print-patterns",		5,	opt_print_patterns},
+	{"no-print-ISBN-table",		10,	opt_print_ISBN_table},
+	{"no-print-patterns",		10,	opt_print_patterns},
 	{"no-read-init-files",		6,	opt_read_init_files},
 	{"no-trace-file-opening",	4,	opt_trace_file_opening},
-	{"print-patterns",		2,	opt_print_patterns},
+	{"print-ISBN-table",		7,	opt_print_ISBN_table},
+	{"print-patterns",		7,	opt_print_patterns},
 	{"read-init-files",		3,	opt_read_init_files},
 	{"trace-file-opening",		1,	opt_trace_file_opening},
 	{(const char*)NULL,		0,	(void (*)(VOID))NULL},
@@ -220,10 +293,12 @@ char *argv[];
 	/* Do argument scan for options that must be known BEFORE
 	initializations are attempted. */
 
-	if ( (argv[k][1] != '\0') && isoptionprefix(argv[k][0]) )
+	if ( (argv[k][1] != '\0') && is_optionprefix((int)argv[k][0]) )
 	{				/* then process command-line switch */
 	    current_index = k;
 	    current_option = argv[k];
+	    if (is_optionprefix((int)current_option[1]))
+		current_option++;	/* allow GNU/POSIX --option */
 	    next_option = argv[k+1];
 	    (void)apply_function(current_option+1,options);
 	}
@@ -232,28 +307,28 @@ char *argv[];
 
 
 #if defined(HAVE_STDC)
-int
-isoptionprefix(int c)
+bool
+is_optionprefix(int c)
 #else /* K&R style */
-int
-isoptionprefix(c)
+bool
+is_optionprefix(c)
 int c;
 #endif
 {
-    int result;
+    bool result;
 
-    result = 0;
+    result = false;
 
 #if OS_PCDOS
-    result = (((c) == '-') || ((c) == '/'));
+    result = (((c) == (int)'-') || ((c) == (int)'/')) ? true : false;
 #endif /* OS_PCDOS */
 
 #if OS_UNIX
-    result = ((c) == '-');
+    result = ((c) == (int)'-') ? true : false;
 #endif /* OS_UNIX */
 
 #if OS_VAXVMS
-    result = (((c) == '-') || ((c) == '/'));
+    result = (((c) == (int)'-') || ((c) == (int)'/')) ? true : false;
 #endif /* OS_VAXVMS */
 
     return (result);
@@ -274,22 +349,26 @@ opt_author(VOID)
     {
 	"Author:\n",
 	"\tNelson H. F. Beebe\n",
-	"\tCenter for Scientific Computing\n",
 	"\tUniversity of Utah\n",
-	"\tDepartment of Mathematics, 322 INSCC\n",
+	"\tDepartment of Mathematics, 110 LCB\n",
 	"\t155 S 1400 E RM 233\n",
 	"\tSalt Lake City, UT 84112-0090\n",
 	"\tUSA\n",
 	"\tTel: +1 801 581 5254\n",
-	"\tFAX: +1 801 585 1640, +1 801 581 4148\n",
-	"\tEmail: beebe@math.utah.edu, beebe@acm.org, beebe@ieee.org (Internet)\n",
+	"\tFAX: +1 801 581 4148\n",
+	"\tEmail: beebe@math.utah.edu, beebe@acm.org, beebe@computer.org (Internet)\n",
 	(const char*)NULL,
     };
 
     out_lines(stdlog, author, NO);
-    exit(EXIT_SUCCESS);
+    finish(EXIT_SUCCESS);
 }
 
+static void
+opt_brace_protect(VOID)
+{
+    brace_protect = YESorNOarg();
+}
 
 static void
 opt_check_values(VOID)
@@ -298,6 +377,29 @@ opt_check_values(VOID)
 }
 
 
+static void
+opt_copyleft(VOID)
+{
+    static CONST char *copyleft[] =
+    {
+	"Copyright (C) 1990--2021 by Nelson H. F. Beebe and the Free Software Foundation, Inc.\n",
+	"This is free software; see the source for copying conditions.  There is NO\n",
+	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
+	(const char*)NULL,
+    };
+
+    out_lines(stdlog, copyleft, NO);
+    finish(EXIT_SUCCESS);
+}
+
+
+static void
+opt_debug_match_failures(VOID)
+{
+    debug_match_failures = YESorNOarg();
+}
+
+
 static void
 opt_delete_empty_values(VOID)
 {
@@ -309,11 +411,12 @@ static void
 opt_error_log(VOID)
 {
     current_index++;
-    if ((stdlog = tfopen(next_option,"w")) == (FILE*)NULL)
+
+    if ((stdlog = tfopen(next_option,"w"), stdlog) == (FILE*)NULL)
     {
-	fprintf(stderr, "%s cannot open error log file [%s]",
-	    WARNING_PREFIX, next_option);
-	fprintf(stderr, " -- using stderr instead\n");
+	(void)fprintf(stderr, "%s cannot open error log file [%s]",
+		      WARNING_PREFIX, next_option);
+	(void)fprintf(stderr, " -- using stderr instead\n");
 	perror("perror() says");
 	stdlog = stderr;
     }
@@ -329,6 +432,27 @@ opt_file_position(VOID)
 }
 
 
+static void
+opt_fix_accents(VOID)
+{
+    fix_accents = YESorNOarg();
+}
+
+
+static void
+opt_fix_braces(VOID)
+{
+    fix_braces = YESorNOarg();
+}
+
+
+static void
+opt_fix_degrees(VOID)
+{
+    fix_degrees = YESorNOarg();
+}
+
+
 static void
 opt_fix_font_changes(VOID)
 {
@@ -343,6 +467,13 @@ opt_fix_initials(VOID)
 }
 
 
+static void
+opt_fix_math(VOID)
+{
+    fix_math = YESorNOarg();
+}
+
+
 static void
 opt_fix_names(VOID)
 {
@@ -350,34 +481,19 @@ opt_fix_names(VOID)
 }
 
 
+static void
+opt_German_style(VOID)
+{
+    German_style = YESorNOarg();
+}
+
+
 static void
 opt_help(VOID)
 {
-    static CONST char *help_lines[] =
-    {
-	"\nUsage: ",
-	(const char*)NULL,
-	" [ -author ] [ -error-log filename ] [ -help ] [ '-?' ]\n",
-	"\t[ -init-file filename ] [ -max-width width ]\n",
-	"\t[ -[no-]align-equals ] [ -[no-]check-values ]\n",
-	"\t[ -[no-]delete-empty-values ] [ -[no-]file-position ]\n",
-	"\t[ -[no-]fix-font-changes ] [ -[no-]fix-initials ]\n",
-	"\t[ -[no-]fix-names ] [ -[no-]German-style ]\n",
-	"\t[ -[no-]keep-linebreaks ] [ -[no-]keep-parbreaks ]\n",
-	"\t[ -[no-]keep-preamble-spaces ] [ -[no-]keep-spaces ]\n",
-	"\t[ -[no-]keep-string-spaces ] [ -[no-]parbreaks ]\n",
-	"\t[ -[no-]prettyprint ] [ -[no-]print-patterns ]\n",
-	"\t[ -[no-]read-init-files ] [ -[no-]remove-OPT-prefixes ]\n",
-	"\t[ -[no-]scribe ] [ -[no-]trace-file-opening ]\n",
-	"\t[ -[no-]warnings ] [ -version ]\n",
-	"\t[ <infile or bibfile1 bibfile2 bibfile3 ...] >outfile\n",
-	"\n",
-#include "bibclean.h"
-    };
-
     help_lines[1] = program_name;	/* cannot have this in initializer */
     out_lines(stdlog, help_lines, (screen_lines > 0) ? YES : NO);
-    exit(EXIT_SUCCESS);
+    finish(EXIT_SUCCESS);
 }
 
 
@@ -390,9 +506,10 @@ opt_init_file(VOID)
 
 
 static void
-opt_German_style(VOID)
+opt_ISBN_file(VOID)
 {
-    German_style = YESorNOarg();
+    current_index++;
+    do_ISBN_file((const char*)NULL,next_option);
 }
 
 
@@ -431,6 +548,14 @@ opt_keep_string_spaces(VOID)
 }
 
 
+static void
+opt_keyword_file(VOID)
+{
+    current_index++;
+    do_keyword_file((const char*)NULL,next_option);
+}
+
+
 static void
 opt_max_width(VOID)
 {
@@ -441,6 +566,19 @@ opt_max_width(VOID)
 }
 
 
+static void
+opt_output_file(VOID)
+{
+    current_index++;
+
+    if (freopen(next_option, "w", stdout) != stdout)
+    {
+	(void)fprintf(stderr, "Cannot open file [%s] for output: job terminated!\n", next_option);
+	finish(EXIT_FAILURE);
+    }
+}
+
+
 static void
 opt_parbreaks(VOID)
 {
@@ -455,6 +593,20 @@ opt_prettyprint(VOID)
 }
 
 
+static void
+opt_print_ISBN_table(VOID)
+{
+    print_ISBN_table = YESorNOarg();
+}
+
+
+static void
+opt_print_keyword_table(VOID)
+{
+    print_keyword_table = YESorNOarg();
+}
+
+
 static void
 opt_print_patterns(VOID)
 {
@@ -462,6 +614,13 @@ opt_print_patterns(VOID)
 }
 
 
+static void
+opt_quiet(VOID)
+{
+    warnings = (YESorNOarg() == YES) ? NO : YES;
+}
+
+
 static void
 opt_read_init_files(VOID)
 {
@@ -494,7 +653,7 @@ static void
 opt_version(VOID)
 {
     version();
-    exit(EXIT_SUCCESS);
+    finish(EXIT_SUCCESS);
 }
 
 
@@ -508,30 +667,9 @@ opt_warnings(VOID)
 static void
 usage(VOID)
 {
-    static CONST char *usage_lines[] =
-    {
-	"\nUsage: ",
-	(const char*)NULL,
-	" [ -author ] [ -error-log filename ] [ -help ] [ '-?' ]\n",
-	"\t[ -init-file filename ] [ -max-width width ]\n",
-	"\t[ -[no-]align-equals ] [ -[no-]check-values ]\n",
-	"\t[ -[no-]delete-empty-values ] [ -[no-]file-position ]\n",
-	"\t[ -[no-]fix-font-changes ] [ -[no-]fix-initials ]\n",
-	"\t[ -[no-]fix-names ] [ -[no-]German-style ]\n",
-	"\t[ -[no-]keep-linebreaks ] [ -[no-]keep-parbreaks ]\n",
-	"\t[ -[no-]keep-preamble-spaces ] [ -[no-]keep-spaces ]\n",
-	"\t[ -[no-]keep-string-spaces ] [ -[no-]parbreaks ]\n",
-	"\t[ -[no-]prettyprint ] [ -[no-]print-patterns ]\n",
-	"\t[ -[no-]read-init-files ] [ -[no-]remove-OPT-prefixes ]\n",
-	"\t[ -[no-]scribe ] [ -[no-]trace-file-opening ]\n",
-	"\t[ -[no-]warnings ] [ -version ]\n",
-	"\t[ <infile or bibfile1 bibfile2 bibfile3 ...] >outfile\n",
-	(const char*)NULL,
-    };
-
     version();
-    usage_lines[1] = program_name;	/* cannot have this in initializer */
-    out_lines(stdlog, usage_lines, NO);
+    help_lines[1] = program_name;	/* cannot have this in initializer */
+    out_lines(stdlog, help_lines, NO);
 }
 
 
@@ -540,21 +678,16 @@ version(VOID)
 {
     static CONST char *version_string[] =
     {
-	BIBCLEAN_VERSION,
+	PACKAGE_NAME, " Version ", PACKAGE_VERSION, " [", PACKAGE_DATE, "]",
+	"\n",
 	"\n",
 
 #if defined(HOST) || defined(USER) || defined(__DATE__) || defined(__TIME__)
 	"Compiled",
 
-#if defined(USER)
-	" by <", USER,
-
-#if defined(HOST)
-	"@", HOST,
-#endif /* defined(HOST) */
-
-	">",
-#endif /* defined(USER) */
+#if defined(USER) && defined(HOST)	/* suppressable for privacy reasons */
+	" by <", USER, "@", HOST, ">",
+#endif /* defined(USER) && defined(HOST) */
 
 #if defined(__DATE__)
 	" on ", __DATE__,
@@ -578,6 +711,12 @@ version(VOID)
 
 	"\n",
 #endif /* defined(HOST)||defined(USER)||defined(__DATE__)||defined(__TIME__) */
+	"E-mail bug reports to ", PACKAGE_BUGREPORT, "\n",
+
+	"\n",
+	"Copyright (C) 1990--2021 by Nelson H. F. Beebe and the Free Software Foundation, Inc.\n",
+	"This is free software; see the source for copying conditions.  There is NO\n",
+	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
 
 	(const char*)NULL,
     };
