@@ -8,7 +8,7 @@
 #include "xctype.h"
 #include "xstring.h"
 
-RCSID("$Id: strist.c,v 1.1 1995/10/07 15:50:28 beebe Exp beebe $")
+RCSID("$Id: strist.c,v 1.2 2003/08/22 23:23:07 beebe Exp beebe $")
 
 
 /* Return pointer to location of sub[] in s[], ignoring letter case,
@@ -35,7 +35,7 @@ const char  *sub;
 	return ((char*)NULL);
     if ((sub == (char*)NULL) || (*sub == '\0'))
 	return ((char*)s); /* NULL substring always found at start */
-    for (;*s;)
+    for (; (*s != '\0');)
     {
 	if (strnicmp(s,sub,length) == 0)
 	    return ((char*)s);
