@@ -1,6 +1,10 @@
 #ifndef TOKLST_H_DEFINED_
 #define TOKLST_H_DEFINED_
 
+#if defined(__APPLE__)
+#define token_t Token_t	/* avoid conflict with type in /usr/include/bsm/audit.h */
+#endif
+
 #if defined(HAVE_STDC)
 typedef enum token_list {
     TOKEN_UNKNOWN = 0,
