@@ -236,7 +236,7 @@ int	vms_getchar(void);
 int	vms_read(int file_desc_, char *buffer_, int nbytes_);
 long	vms_ungetc(char c_, FILE *fp_);
 
-static char rcsid[] = "$Id: vaxvms.c,v 1.2 1996/05/03 20:23:16 beebe Exp beebe $";
+static char rcsid[] = "$Id: vaxvms.c,v 1.3 2003/08/22 23:25:43 beebe Exp beebe $";
 
 
 /**********************************************************************/
@@ -630,7 +630,7 @@ char  *name;
     p = ucname;
     while (*name)	/* VMS logical names must be upper-case */
     {
-      *p++ = islower(*name) ? toupper(*name) : *name;
+      *p++ = Islower(*name) ? toupper(*name) : *name;
       ++name;
     }
     *p = '\0';
@@ -892,5 +892,5 @@ time_t times[2];
     return (0);
 }
 #else
-static int this_is_not_vms = 0;		/* to provide SOMETHING to compile */
+/*@unused@*/ static int this_is_not_vms = 0;		/* to provide SOMETHING to compile */
 #endif /* VMS */
