@@ -104,7 +104,7 @@
 ** The source is 40 blocks (600 lines) so I am not sure that I should
 ** post the actual source in this group, but I have (see below).  If you
 ** are interested, please look through the documentation, try it out,
-** and send any significant comments, suggestions, useful peices of
+** and send any significant comments, suggestions, useful pieces of
 ** code, etc. to one of the addresses below.	I will post summaries to
 ** comp.os.vms/INFO-VAX and comp.lang.c/INFO-C.  Please excuse the
 ** "lived-in" look of the code but I am still working on this thing so
@@ -201,7 +201,7 @@ static char version[] = /* (Not currently) printed inside "debug" option. */
   "cmd_lin() V2.0, Copyright (C) 1988 Eric Tye McQueen (3-Mar).";
 #endif /* 0 */
 
-static char rcsid[] = "$Id: vmswild.c,v 1.2 1996/05/03 20:09:24 beebe Exp $";
+/*@unused@*/ static char rcsid[] = "$Id: vmswild.c,v 1.5 2013/11/30 19:17:27 beebe Exp beebe $";
 
 
 #ifndef VMS	/* Should be changed to Standard C-conforming name soon. */
@@ -211,14 +211,14 @@ static char rcsid[] = "$Id: vmswild.c,v 1.2 1996/05/03 20:09:24 beebe Exp $";
 char **
 #if defined(HAVE_STDC)
 cmd_lin(
-char	*opt0,				/* Options specified in `main()' */
-int	*ainpc,				/* &main\argc (input and output) */
+/*@unused@*/ char	*opt0,		/* Options specified in `main()' */
+/*@unused@*/ int	*ainpc,		/* &main\argc (input and output) */
 char	**inpv				/* main\argv (input) */
 )
 #else /* NOT defined(HAVE_STDC) */
 cmd_lin( opt0, ainpc, inpv )
-char	*opt0;				/* Options specified in `main()' */
-int	*ainpc;				/* &main\argc (input and output) */
+/*@unused@*/ char	*opt0;		/* Options specified in `main()' */
+/*@unused@*/ int	*ainpc;		/* &main\argc (input and output) */
 char	**inpv;				/* main\argv (input) */
 #endif /* defined(HAVE_STDC) */
 {
@@ -329,7 +329,7 @@ static uint	crelnm(char *table, char *name, char *value);
 static void	*emalloc(unsigned siz);
 static void	*erealloc(void *ptr, unsigned siz);
 static char	*expand_logical(char *logname);
-static bool	iswild(char *name);
+static bool	is_wild(char *name);
 static char	*nameonly(char *file);
 static char	*normalize_filename(char *filename);
 static char	*nxt_wld(char *wild);
@@ -403,7 +403,7 @@ char	**inpv;				/* main\argv (input) */
 			inpv = redirin( ainpc, inpv );
 		else if(  OPredir  &&	 '>' == **inpv	)
 			inpv = redirout( ainpc, inpv );
-		else if(  OPexpand  &&  iswild( *inpv )  )
+		else if(  OPexpand  &&  is_wild( *inpv )  )
 		{
 			if(  ( cp = nxt_wld(*inpv) )  )
 			{
@@ -562,11 +562,11 @@ char	*logname;
 
 static bool	/* Returns 1 if `name' looks like a wild-carded file name. */
 #if defined(HAVE_STDC)
-iswild(
+is_wild(
   char *name
 )
 #else /* NOT defined(HAVE_STDC) */
-iswild( name )
+is_wild( name )
   char *name;
 #endif /* defined(HAVE_STDC) */
 {
@@ -637,7 +637,7 @@ char	*filename;		/* with logical names expanded */
 	q = strcpy(emalloc(strlen(filename)),filename);
 
     for (p = q ; *p; ++p)	/* convert filename to lowercase */
-	*p = isupper(*p) ? tolower(*p) : *p;
+	*p = Isupper(*p) ? tolower(*p) : *p;
 
 				/* remove remnants of rooted logical names */
     for (p = strsub(q,".]["); p; p = strsub(q,".]["))
@@ -710,6 +710,7 @@ nxt_wld( wild ) /* Returns 0 if all matching filenames have been returned. */
     p = normalize_filename(file);
     (void)strcpy(file,p);
     FREE(p);
+    p = (char*)NULL;
     if (strsub(wild,";") == (char*)NULL) /* strip generation numbers too */
     {
 	p = strrchr(file,';');
@@ -772,11 +773,11 @@ read_cmd_lin( ainpc, inpv )
 		midv = add_arg( *(inpv++), &midc, midv );
 	cp = strcpy(  emalloc( strlen(line)+1 ),  line	);
 	do {
-		while(	isspace( *cp )	)
+		while(	Isspace( *cp )	)
 			cp++;
 		if(  !*( ap = cp )  )
 			break;
-		while(	*cp  &&	 !isspace(*cp)	)
+		while(	*cp  &&	 !Isspace(*cp)	)
 			cp++;
 		c = *cp;   *cp++ = '\0';
 		midv = add_arg( ap, &midc, midv );
